@@ -2,7 +2,8 @@
 
 '''SMTP/ESMTP client class.
 
-This should follow RFC 821 (SMTP) and RFC 1869 (ESMTP).
+This should follow RFC 821 (SMTP), RFC 1869 (ESMTP), RFC 2554 (SMTP
+Authentication) and RFC 2487 (Secure SMTP over TLS).
 
 Notes:
 
@@ -36,19 +37,29 @@ Example:
 #     Eric S. Raymond <esr@thyrsus.com>
 # Better RFC 821 compliance (MAIL and RCPT, and CRLF in data)
 #     by Carey Evans <c.evans@clear.net.nz>, for picky mail servers.
-#    
+# RFC 2554 (authentication) support by Gerhard Haering <gerhard@bigfoot.de>.
+#
 # This was modified from the Python 1.5 library HTTP lib.
 
 import socket
-import string
 import re
 import rfc822
-import types
+import base64
+import hmac
+from email.base64MIME import encode as encode_base64
+from sys import stderr
+
+__all__ = ["SMTPException","SMTPServerDisconnected","SMTPResponseException",
+           "SMTPSenderRefused","SMTPRecipientsRefused","SMTPDataError",
+           "SMTPConnectError","SMTPHeloError","SMTPAuthenticationError",
+           "quoteaddr","quotedata","SMTP"]
 
 SMTP_PORT = 25
 CRLF="\r\n"
 
-# Exception classes used by this module. 
+OLDSTYLE_AUTH = re.compile(r"auth=(.*)", re.I)
+
+# Exception classes used by this module.
 class SMTPException(Exception):
     """Base class for all exceptions raised by this module."""
 
@@ -76,6 +87,7 @@ class SMTPResponseException(SMTPException):
 
 class SMTPSenderRefused(SMTPResponseException):
     """Sender address refused.
+
     In addition to the attributes set by on all SMTPResponseException
     exceptions, this sets `sender' to the string that the SMTP refused.
     """
@@ -88,9 +100,10 @@ class SMTPSenderRefused(SMTPResponseException):
 
 class SMTPRecipientsRefused(SMTPException):
     """All recipient addresses refused.
+
     The errors for each recipient are accessible through the attribute
-    'recipients', which is a dictionary of exactly the same sort as 
-    SMTP.sendmail() returns.  
+    'recipients', which is a dictionary of exactly the same sort as
+    SMTP.sendmail() returns.
     """
 
     def __init__(self, recipients):
@@ -107,20 +120,66 @@ class SMTPConnectError(SMTPResponseException):
 class SMTPHeloError(SMTPResponseException):
     """The server refused our HELO reply."""
 
+class SMTPAuthenticationError(SMTPResponseException):
+    """Authentication error.
+
+    Most probably the server didn't accept the username/password
+    combination provided.
+    """
+
+class SSLFakeSocket:
+    """A fake socket object that really wraps a SSLObject.
+
+    It only supports what is needed in smtplib.
+    """
+    def __init__(self, realsock, sslobj):
+        self.realsock = realsock
+        self.sslobj = sslobj
+
+    def send(self, str):
+        self.sslobj.write(str)
+        return len(str)
+
+    sendall = send
+
+    def close(self):
+        self.realsock.close()
+
+class SSLFakeFile:
+    """A fake file like object that really wraps a SSLObject.
+
+    It only supports what is needed in smtplib.
+    """
+    def __init__( self, sslobj):
+        self.sslobj = sslobj
+
+    def readline(self):
+        str = ""
+        chr = None
+        while chr != "\n":
+            chr = self.sslobj.read(1)
+            str += chr
+        return str
+
+    def close(self):
+        pass
 
 def quoteaddr(addr):
     """Quote a subset of the email addresses defined by RFC 821.
 
     Should be able to handle anything rfc822.parseaddr can handle.
     """
-    m=None
+    m = (None, None)
     try:
         m=rfc822.parseaddr(addr)[1]
     except AttributeError:
         pass
-    if not m:
-        #something weird here.. punt -ddm
-        return addr
+    if m == (None, None): # Indicates parse failure or AttributeError
+        # something weird here.. punt -ddm
+        return "<%s>" % addr
+    elif m is None:
+        # the sender wants an empty return address
+        return "<>"
     else:
         return "<%s>" % m
 
@@ -137,27 +196,27 @@ def quotedata(data):
 class SMTP:
     """This class manages a connection to an SMTP or ESMTP server.
     SMTP Objects:
-        SMTP objects have the following attributes:    
-            helo_resp 
-                This is the message given by the server in response to the 
+        SMTP objects have the following attributes:
+            helo_resp
+                This is the message given by the server in response to the
                 most recent HELO command.
-                
+
             ehlo_resp
-                This is the message given by the server in response to the 
+                This is the message given by the server in response to the
                 most recent EHLO command. This is usually multiline.
 
-            does_esmtp 
+            does_esmtp
                 This is a True value _after you do an EHLO command_, if the
                 server supports ESMTP.
 
-            esmtp_features 
+            esmtp_features
                 This is a dictionary, which, if the server supports ESMTP,
                 will _after you do an EHLO command_, contain the names of the
                 SMTP service extensions this server supports, and their
                 parameters (if any).
 
-                Note, all extension names are mapped to lower case in the 
-                dictionary. 
+                Note, all extension names are mapped to lower case in the
+                dictionary.
 
         See each method's docstrings for details.  In general, there is a
         method of the same name to perform each SMTP command.  There is also a
@@ -169,13 +228,15 @@ class SMTP:
     ehlo_resp = None
     does_esmtp = 0
 
-    def __init__(self, host = '', port = 0):
+    def __init__(self, host = '', port = 0, local_hostname = None):
         """Initialize a new instance.
 
         If specified, `host' is the name of the remote host to which to
         connect.  If specified, `port' specifies the port to which to connect.
         By default, smtplib.SMTP_PORT is used.  An SMTPConnectError is raised
-        if the specified `host' doesn't respond correctly.
+        if the specified `host' doesn't respond correctly.  If specified,
+        `local_hostname` is used as the FQDN of the local host.  By default,
+        the local hostname is found using socket.getfqdn().
 
         """
         self.esmtp_features = {}
@@ -183,7 +244,20 @@ class SMTP:
             (code, msg) = self.connect(host, port)
             if code != 220:
                 raise SMTPConnectError(code, msg)
-    
+        if local_hostname is not None:
+            self.local_hostname = local_hostname
+        else:
+            # RFC 2821 says we should use the fqdn in the EHLO/HELO verb, and
+            # if that can't be calculated, that we should use a domain literal
+            # instead (essentially an encoded IP address like [A.B.C.D]).
+            fqdn = socket.getfqdn()
+            if '.' in fqdn:
+                self.local_hostname = fqdn
+            else:
+                # We can't find an fqdn hostname, so use a domain literal
+                addr = socket.gethostbyname(socket.gethostname())
+                self.local_hostname = '[%s]' % addr
+
     def set_debuglevel(self, debuglevel):
         """Set the debug output level.
 
@@ -204,38 +278,48 @@ class SMTP:
         specified during instantiation.
 
         """
-        if not port:
-            i = string.find(host, ':')
+        if not port and (host.find(':') == host.rfind(':')):
+            i = host.rfind(':')
             if i >= 0:
                 host, port = host[:i], host[i+1:]
-                try: port = string.atoi(port)
-                except string.atoi_error:
+                try: port = int(port)
+                except ValueError:
                     raise socket.error, "nonnumeric port"
         if not port: port = SMTP_PORT
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        if self.debuglevel > 0: print 'connect:', (host, port)
-        try:
-            self.sock.connect((host, port))
-        except socket.error:
-            self.close()
-            raise
-        (code,msg)=self.getreply()
-        if self.debuglevel >0 : print "connect:", msg
-        return (code,msg)
-    
+        if self.debuglevel > 0: print>>stderr, 'connect:', (host, port)
+        msg = "getaddrinfo returns an empty list"
+        self.sock = None
+        for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
+            af, socktype, proto, canonname, sa = res
+            try:
+                self.sock = socket.socket(af, socktype, proto)
+                if self.debuglevel > 0: print>>stderr, 'connect:', (host, port)
+                self.sock.connect(sa)
+            except socket.error, msg:
+                if self.debuglevel > 0: print>>stderr, 'connect fail:', (host, port)
+                if self.sock:
+                    self.sock.close()
+                self.sock = None
+                continue
+            break
+        if not self.sock:
+            raise socket.error, msg
+        (code, msg) = self.getreply()
+        if self.debuglevel > 0: print>>stderr, "connect:", msg
+        return (code, msg)
+
     def send(self, str):
         """Send `str' to the server."""
-        if self.debuglevel > 0: print 'send:', `str`
+        if self.debuglevel > 0: print>>stderr, 'send:', repr(str)
         if self.sock:
             try:
-                sendptr = 0
-                while sendptr < len(str):
-                    sendptr = sendptr + self.sock.send(str[sendptr:])
+                self.sock.sendall(str)
             except socket.error:
+                self.close()
                 raise SMTPServerDisconnected('Server not connected')
         else:
             raise SMTPServerDisconnected('please run connect() first')
- 
+
     def putcmd(self, cmd, args=""):
         """Send a command to the server."""
         if args == "":
@@ -243,10 +327,10 @@ class SMTP:
         else:
             str = '%s %s%s' % (cmd, args, CRLF)
         self.send(str)
-    
+
     def getreply(self):
         """Get a reply from the server.
-        
+
         Returns a tuple consisting of:
 
           - server response code (e.g. '250', or such, if all goes well)
@@ -265,13 +349,13 @@ class SMTP:
             if line == '':
                 self.close()
                 raise SMTPServerDisconnected("Connection unexpectedly closed")
-            if self.debuglevel > 0: print 'reply:', `line`
-            resp.append(string.strip(line[4:]))
+            if self.debuglevel > 0: print>>stderr, 'reply:', repr(line)
+            resp.append(line[4:].strip())
             code=line[:3]
             # Check that the error code is syntactically correct.
             # Don't attempt to read a continuation line if it is broken.
             try:
-                errcode = string.atoi(code)
+                errcode = int(code)
             except ValueError:
                 errcode = -1
                 break
@@ -279,11 +363,11 @@ class SMTP:
             if line[3:4]!="-":
                 break
 
-        errmsg = string.join(resp,"\n")
-        if self.debuglevel > 0: 
-            print 'reply: retcode (%s); Msg: %s' % (errcode,errmsg)
+        errmsg = "\n".join(resp)
+        if self.debuglevel > 0:
+            print>>stderr, 'reply: retcode (%s); Msg: %s' % (errcode,errmsg)
         return errcode, errmsg
-    
+
     def docmd(self, cmd, args=""):
         """Send a command, and return its response code."""
         self.putcmd(cmd,args)
@@ -295,10 +379,7 @@ class SMTP:
         Hostname to send for this command defaults to the FQDN of the local
         host.
         """
-        if name:
-            self.putcmd("helo", name)
-        else:
-            self.putcmd("helo", socket.getfqdn())
+        self.putcmd("helo", name or self.local_hostname)
         (code,msg)=self.getreply()
         self.helo_resp=msg
         return (code,msg)
@@ -308,34 +389,54 @@ class SMTP:
         Hostname to send for this command defaults to the FQDN of the local
         host.
         """
-        if name:
-            self.putcmd("ehlo", name)
-        else:
-            self.putcmd("ehlo", socket.getfqdn())
+        self.esmtp_features = {}
+        self.putcmd("ehlo", name or self.local_hostname)
         (code,msg)=self.getreply()
-        # According to RFC1869 some (badly written) 
-        # MTA's will disconnect on an ehlo. Toss an exception if 
+        # According to RFC1869 some (badly written)
+        # MTA's will disconnect on an ehlo. Toss an exception if
         # that happens -ddm
         if code == -1 and len(msg) == 0:
+            self.close()
             raise SMTPServerDisconnected("Server not connected")
         self.ehlo_resp=msg
-        if code<>250:
+        if code != 250:
             return (code,msg)
         self.does_esmtp=1
         #parse the ehlo response -ddm
-        resp=string.split(self.ehlo_resp,'\n')
+        resp=self.ehlo_resp.split('\n')
         del resp[0]
         for each in resp:
-            m=re.match(r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*)',each)
+            # To be able to communicate with as many SMTP servers as possible,
+            # we have to take the old-style auth advertisement into account,
+            # because:
+            # 1) Else our SMTP feature parser gets confused.
+            # 2) There are some servers that only advertise the auth methods we
+            #    support using the old style.
+            auth_match = OLDSTYLE_AUTH.match(each)
+            if auth_match:
+                # This doesn't remove duplicates, but that's no problem
+                self.esmtp_features["auth"] = self.esmtp_features.get("auth", "") \
+                        + " " + auth_match.groups(0)[0]
+                continue
+
+            # RFC 1869 requires a space between ehlo keyword and parameters.
+            # It's actually stricter, in that only spaces are allowed between
+            # parameters, but were not going to check for that here.  Note
+            # that the space isn't present if there are no parameters.
+            m=re.match(r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*) ?',each)
             if m:
-                feature=string.lower(m.group("feature"))
-                params=string.strip(m.string[m.end("feature"):])
-                self.esmtp_features[feature]=params
+                feature=m.group("feature").lower()
+                params=m.string[m.end("feature"):].strip()
+                if feature == "auth":
+                    self.esmtp_features[feature] = self.esmtp_features.get(feature, "") \
+                            + " " + params
+                else:
+                    self.esmtp_features[feature]=params
         return (code,msg)
 
     def has_extn(self, opt):
         """Does the server support a given SMTP service extension?"""
-        return self.esmtp_features.has_key(string.lower(opt))
+        return opt.lower() in self.esmtp_features
 
     def help(self, args=''):
         """SMTP 'help' command.
@@ -355,7 +456,7 @@ class SMTP:
         """SMTP 'mail' command -- begins mail xfer session."""
         optionlist = ''
         if options and self.does_esmtp:
-            optionlist = ' ' + string.join(options, ' ')
+            optionlist = ' ' + ' '.join(options)
         self.putcmd("mail", "FROM:%s%s" % (quoteaddr(sender) ,optionlist))
         return self.getreply()
 
@@ -363,12 +464,12 @@ class SMTP:
         """SMTP 'rcpt' command -- indicates 1 recipient for this mail."""
         optionlist = ''
         if options and self.does_esmtp:
-            optionlist = ' ' + string.join(options, ' ')
+            optionlist = ' ' + ' '.join(options)
         self.putcmd("rcpt","TO:%s%s" % (quoteaddr(recip),optionlist))
         return self.getreply()
 
     def data(self,msg):
-        """SMTP 'DATA' command -- sends message data to server. 
+        """SMTP 'DATA' command -- sends message data to server.
 
         Automatically quotes lines beginning with a period per rfc821.
         Raises SMTPDataError if there is an unexpected reply to the
@@ -377,8 +478,8 @@ class SMTP:
         """
         self.putcmd("data")
         (code,repl)=self.getreply()
-        if self.debuglevel >0 : print "data:", (code,repl)
-        if code <> 354:
+        if self.debuglevel >0 : print>>stderr, "data:", (code,repl)
+        if code != 354:
             raise SMTPDataError(code,repl)
         else:
             q = quotedata(msg)
@@ -387,7 +488,7 @@ class SMTP:
             q = q + "." + CRLF
             self.send(q)
             (code,msg)=self.getreply()
-            if self.debuglevel >0 : print "data:", (code,msg)
+            if self.debuglevel >0 : print>>stderr, "data:", (code,msg)
             return (code,msg)
 
     def verify(self, address):
@@ -403,15 +504,114 @@ class SMTP:
         return self.getreply()
 
     # some useful methods
-    def sendmail(self, from_addr, to_addrs, msg, mail_options=[],
-                 rcpt_options=[]): 
-        """This command performs an entire mail transaction. 
 
-        The arguments are: 
+    def login(self, user, password):
+        """Log in on an SMTP server that requires authentication.
+
+        The arguments are:
+            - user:     The user name to authenticate with.
+            - password: The password for the authentication.
+
+        If there has been no previous EHLO or HELO command this session, this
+        method tries ESMTP EHLO first.
+
+        This method will return normally if the authentication was successful.
+
+        This method may raise the following exceptions:
+
+         SMTPHeloError            The server didn't reply properly to
+                                  the helo greeting.
+         SMTPAuthenticationError  The server didn't accept the username/
+                                  password combination.
+         SMTPException            No suitable authentication method was
+                                  found.
+        """
+
+        def encode_cram_md5(challenge, user, password):
+            challenge = base64.decodestring(challenge)
+            response = user + " " + hmac.HMAC(password, challenge).hexdigest()
+            return encode_base64(response, eol="")
+
+        def encode_plain(user, password):
+            return encode_base64("%s\0%s\0%s" % (user, user, password), eol="")
+
+
+        AUTH_PLAIN = "PLAIN"
+        AUTH_CRAM_MD5 = "CRAM-MD5"
+        AUTH_LOGIN = "LOGIN"
+
+        if self.helo_resp is None and self.ehlo_resp is None:
+            if not (200 <= self.ehlo()[0] <= 299):
+                (code, resp) = self.helo()
+                if not (200 <= code <= 299):
+                    raise SMTPHeloError(code, resp)
+
+        if not self.has_extn("auth"):
+            raise SMTPException("SMTP AUTH extension not supported by server.")
+
+        # Authentication methods the server supports:
+        authlist = self.esmtp_features["auth"].split()
+
+        # List of authentication methods we support: from preferred to
+        # less preferred methods. Except for the purpose of testing the weaker
+        # ones, we prefer stronger methods like CRAM-MD5:
+        preferred_auths = [AUTH_CRAM_MD5, AUTH_PLAIN, AUTH_LOGIN]
+
+        # Determine the authentication method we'll use
+        authmethod = None
+        for method in preferred_auths:
+            if method in authlist:
+                authmethod = method
+                break
+
+        if authmethod == AUTH_CRAM_MD5:
+            (code, resp) = self.docmd("AUTH", AUTH_CRAM_MD5)
+            if code == 503:
+                # 503 == 'Error: already authenticated'
+                return (code, resp)
+            (code, resp) = self.docmd(encode_cram_md5(resp, user, password))
+        elif authmethod == AUTH_PLAIN:
+            (code, resp) = self.docmd("AUTH",
+                AUTH_PLAIN + " " + encode_plain(user, password))
+        elif authmethod == AUTH_LOGIN:
+            (code, resp) = self.docmd("AUTH",
+                "%s %s" % (AUTH_LOGIN, encode_base64(user, eol="")))
+            if code != 334:
+                raise SMTPAuthenticationError(code, resp)
+            (code, resp) = self.docmd(encode_base64(password, eol=""))
+        elif authmethod is None:
+            raise SMTPException("No suitable authentication method found.")
+        if code not in [235, 503]:
+            # 235 == 'Authentication successful'
+            # 503 == 'Error: already authenticated'
+            raise SMTPAuthenticationError(code, resp)
+        return (code, resp)
+
+    def starttls(self, keyfile = None, certfile = None):
+        """Puts the connection to the SMTP server into TLS mode.
+
+        If the server supports TLS, this will encrypt the rest of the SMTP
+        session. If you provide the keyfile and certfile parameters,
+        the identity of the SMTP server and client can be checked. This,
+        however, depends on whether the socket module really checks the
+        certificates.
+        """
+        (resp, reply) = self.docmd("STARTTLS")
+        if resp == 220:
+            sslobj = socket.ssl(self.sock, keyfile, certfile)
+            self.sock = SSLFakeSocket(self.sock, sslobj)
+            self.file = SSLFakeFile(sslobj)
+        return (resp, reply)
+
+    def sendmail(self, from_addr, to_addrs, msg, mail_options=[],
+                 rcpt_options=[]):
+        """This command performs an entire mail transaction.
+
+        The arguments are:
             - from_addr    : The address sending this mail.
             - to_addrs     : A list of addresses to send this mail to.  A bare
                              string will be treated as a list with 1 address.
-            - msg          : The message to send. 
+            - msg          : The message to send.
             - mail_options : List of ESMTP options (such as 8bitmime) for the
                              mail command.
             - rcpt_options : List of ESMTP options (such as DSN commands) for
@@ -430,7 +630,7 @@ class SMTP:
         This method may raise the following exceptions:
 
          SMTPHeloError          The server didn't reply properly to
-                                the helo greeting. 
+                                the helo greeting.
          SMTPRecipientsRefused  The server rejected ALL recipients
                                 (no mail was sent).
          SMTPSenderRefused      The server didn't accept the from_addr.
@@ -441,11 +641,11 @@ class SMTP:
         Note: the connection will be open even after an exception is raised.
 
         Example:
-      
+
          >>> import smtplib
          >>> s=smtplib.SMTP("localhost")
          >>> tolist=["one@one.org","two@two.org","three@three.org","four@four.org"]
-         >>> msg = '''
+         >>> msg = '''\\
          ... From: Me@my.org
          ... Subject: testin'...
          ...
@@ -453,7 +653,7 @@ class SMTP:
          >>> s.sendmail("me@my.org",tolist,msg)
          { "three@three.org" : ( 550 ,"User unknown" ) }
          >>> s.quit()
-        
+
         In the above example, the message was accepted for delivery to three
         of the four addresses, and one was rejected, with the error code
         550.  If all addresses are accepted, then the method will return an
@@ -470,31 +670,31 @@ class SMTP:
             # Hmmm? what's this? -ddm
             # self.esmtp_features['7bit']=""
             if self.has_extn('size'):
-                esmtp_opts.append("size=" + `len(msg)`)
+                esmtp_opts.append("size=%d" % len(msg))
             for option in mail_options:
                 esmtp_opts.append(option)
 
         (code,resp) = self.mail(from_addr, esmtp_opts)
-        if code <> 250:
+        if code != 250:
             self.rset()
             raise SMTPSenderRefused(code, resp, from_addr)
         senderrs={}
-        if type(to_addrs) == types.StringType:
+        if isinstance(to_addrs, basestring):
             to_addrs = [to_addrs]
         for each in to_addrs:
             (code,resp)=self.rcpt(each, rcpt_options)
-            if (code <> 250) and (code <> 251):
+            if (code != 250) and (code != 251):
                 senderrs[each]=(code,resp)
         if len(senderrs)==len(to_addrs):
             # the server refused all our recipients
             self.rset()
             raise SMTPRecipientsRefused(senderrs)
-        (code,resp)=self.data(msg)
-        if code <> 250:
+        (code,resp) = self.data(msg)
+        if code != 250:
             self.rset()
             raise SMTPDataError(code, resp)
         #if we got here then somebody got our mail
-        return senderrs         
+        return senderrs
 
 
     def close(self):
@@ -516,14 +716,14 @@ class SMTP:
 # Test the sendmail method, which tests most of the others.
 # Note: This always sends to localhost.
 if __name__ == '__main__':
-    import sys, rfc822
+    import sys
 
     def prompt(prompt):
         sys.stdout.write(prompt + ": ")
-        return string.strip(sys.stdin.readline())
+        return sys.stdin.readline().strip()
 
     fromaddr = prompt("From")
-    toaddrs  = string.splitfields(prompt("To"), ',')
+    toaddrs  = prompt("To").split(',')
     print "Enter message, end with ^D:"
     msg = ''
     while 1:
@@ -531,7 +731,7 @@ if __name__ == '__main__':
         if not line:
             break
         msg = msg + line
-    print "Message length is " + `len(msg)`
+    print "Message length is %d" % len(msg)
 
     server = SMTP('localhost')
     server.set_debuglevel(1)
