@@ -26,7 +26,7 @@ typedef __int64 hs_time;
 #ifndef HAVE_GETTIMEOFDAY
 #error "This module requires gettimeofday() on non-Windows platforms!"
 #endif
-#if defined(macintosh) || (defined(PYOS_OS2) && defined(PYCC_GCC))
+#if (defined(PYOS_OS2) && defined(PYCC_GCC))
 #include <sys/time.h>
 #else
 #include <sys/resource.h>
@@ -47,10 +47,6 @@ typedef struct timeval hs_time;
 
 #define BUFFERSIZE 10240
 
-#ifdef macintosh
-#define PATH_MAX 254
-#endif
-
 #if defined(PYOS_OS2) && defined(PYCC_GCC)
 #define PATH_MAX 260
 #endif
@@ -477,6 +473,8 @@ restart:
     }
     else if (!err) {
         result = PyTuple_New(4);
+        if (result == NULL)
+            return NULL;
         PyTuple_SET_ITEM(result, 0, PyInt_FromLong(what));
         PyTuple_SET_ITEM(result, 2, PyInt_FromLong(fileno));
         if (s1 == NULL)
@@ -832,12 +830,14 @@ get_tdelta(ProfilerObject *self)
 
     GETTIMEOFDAY(&tv);
 
-    if (tv.tv_sec == self->prev_timeofday.tv_sec)
-        tdelta = tv.tv_usec - self->prev_timeofday.tv_usec;
-    else
-        tdelta = ((tv.tv_sec - self->prev_timeofday.tv_sec) * 1000000
-                  + tv.tv_usec);
+    tdelta = tv.tv_usec - self->prev_timeofday.tv_usec;
+    if (tv.tv_sec != self->prev_timeofday.tv_sec)
+        tdelta += (tv.tv_sec - self->prev_timeofday.tv_sec) * 1000000;
 #endif
+    /* time can go backwards on some multiprocessor systems or by NTP */
+    if (tdelta < 0)
+        return 0;
+
     self->prev_timeofday = tv;
     return tdelta;
 }
@@ -845,38 +845,6 @@ get_tdelta(ProfilerObject *self)
 
 /* The workhorse:  the profiler callback function. */
 
-static int
-profiler_callback(ProfilerObject *self, PyFrameObject *frame, int what,
-                  PyObject *arg)
-{
-    int tdelta = -1;
-    int fileno;
-
-    if (self->frametimings)
-        tdelta = get_tdelta(self);
-    switch (what) {
-    case PyTrace_CALL:
-        fileno = get_fileno(self, frame->f_code);
-        if (fileno < 0)
-            return -1;
-        if (pack_enter(self, fileno, tdelta,
-                       frame->f_code->co_firstlineno) < 0)
-            return -1;
-        break;
-    case PyTrace_RETURN:
-        if (pack_exit(self, tdelta) < 0)
-            return -1;
-        break;
-    default:
-        /* should never get here */
-        break;
-    }
-    return 0;
-}
-
-
-/* Alternate callback when we want PyTrace_LINE events */
-
 static int
 tracer_callback(ProfilerObject *self, PyFrameObject *frame, int what,
                 PyObject *arg)
@@ -895,7 +863,7 @@ tracer_callback(ProfilerObject *self, PyFrameObject *frame, int what,
     case PyTrace_RETURN:
         return pack_exit(self, get_tdelta(self));
 
-    case PyTrace_LINE:
+    case PyTrace_LINE:  /* we only get these events if we asked for them */
         if (self->linetimings)
             return pack_lineno_tdelta(self, frame->f_lineno,
 				      get_tdelta(self));
@@ -948,7 +916,7 @@ calibrate(void)
         }
 #endif
     }
-#if defined(MS_WINDOWS) || defined(macintosh) || defined(PYOS_OS2) || \
+#if defined(MS_WINDOWS) || defined(PYOS_OS2) || \
     defined(__VMS)
     rusage_diff = -1;
 #else
@@ -989,7 +957,7 @@ do_start(ProfilerObject *self)
     if (self->lineevents)
         PyEval_SetTrace((Py_tracefunc) tracer_callback, (PyObject *)self);
     else
-        PyEval_SetProfile((Py_tracefunc) profiler_callback, (PyObject *)self);
+        PyEval_SetProfile((Py_tracefunc) tracer_callback, (PyObject *)self);
 }
 
 static void
@@ -1425,7 +1393,7 @@ hotshot_logreader(PyObject *unused, PyObject *args)
 static char *
 get_version_string(void)
 {
-    static char *rcsid = "$Revision: 1.34.10.1 $";
+    static char *rcsid = "$Revision: 43172 $";
     char *rev = rcsid;
     char *buffer;
     int i = 0;
@@ -1488,6 +1456,10 @@ write_header(ProfilerObject *self)
                   getcwd(cwdbuffer, sizeof cwdbuffer));
 
     temp = PySys_GetObject("path");
+    if (temp == NULL || !PyList_Check(temp)) {
+        PyErr_SetString(PyExc_RuntimeError, "sys.path must be a list");
+        return -1;
+    }
     len = PyList_GET_SIZE(temp);
     for (i = 0; i < len; ++i) {
         PyObject *item = PyList_GET_ITEM(temp, i);
@@ -1552,9 +1524,11 @@ hotshot_profiler(PyObject *unused, PyObject *args)
             calibrate();
             calibrate();
         }
-        if (write_header(self))
+        if (write_header(self)) {
             /* some error occurred, exception has been set */
+            Py_DECREF(self);
             self = NULL;
+        }
     }
     return (PyObject *) self;
 }
