@@ -1013,6 +1013,8 @@ audioop_ratecv(PyObject *self, PyObject *args)
 		while (d < 0) {
 			if (len == 0) {
 				samps = PyTuple_New(nchannels);
+				if (samps == NULL)
+					goto exit;
 				for (chan = 0; chan < nchannels; chan++)
 					PyTuple_SetItem(samps, chan,
 						Py_BuildValue("(ii)",
@@ -1343,33 +1345,33 @@ audioop_adpcm2lin(PyObject *self, PyObject *args)
 }
 
 static PyMethodDef audioop_methods[] = {
-	{ "max", audioop_max },
-	{ "minmax", audioop_minmax },
-	{ "avg", audioop_avg },
-	{ "maxpp", audioop_maxpp },
-	{ "avgpp", audioop_avgpp },
-	{ "rms", audioop_rms },
-	{ "findfit", audioop_findfit },
-	{ "findmax", audioop_findmax },
-	{ "findfactor", audioop_findfactor },
-	{ "cross", audioop_cross },
-	{ "mul", audioop_mul },
-	{ "add", audioop_add },
-	{ "bias", audioop_bias },
-	{ "ulaw2lin", audioop_ulaw2lin },
-	{ "lin2ulaw", audioop_lin2ulaw },
-	{ "lin2lin", audioop_lin2lin },
-	{ "adpcm2lin", audioop_adpcm2lin },
-	{ "lin2adpcm", audioop_lin2adpcm },
-	{ "tomono", audioop_tomono },
-	{ "tostereo", audioop_tostereo },
-	{ "getsample", audioop_getsample },
-	{ "reverse", audioop_reverse },
-	{ "ratecv", audioop_ratecv, 1 },
+	{ "max", audioop_max, METH_OLDARGS },
+	{ "minmax", audioop_minmax, METH_OLDARGS },
+	{ "avg", audioop_avg, METH_OLDARGS },
+	{ "maxpp", audioop_maxpp, METH_OLDARGS },
+	{ "avgpp", audioop_avgpp, METH_OLDARGS },
+	{ "rms", audioop_rms, METH_OLDARGS },
+	{ "findfit", audioop_findfit, METH_OLDARGS },
+	{ "findmax", audioop_findmax, METH_OLDARGS },
+	{ "findfactor", audioop_findfactor, METH_OLDARGS },
+	{ "cross", audioop_cross, METH_OLDARGS },
+	{ "mul", audioop_mul, METH_OLDARGS },
+	{ "add", audioop_add, METH_OLDARGS },
+	{ "bias", audioop_bias, METH_OLDARGS },
+	{ "ulaw2lin", audioop_ulaw2lin, METH_OLDARGS },
+	{ "lin2ulaw", audioop_lin2ulaw, METH_OLDARGS },
+	{ "lin2lin", audioop_lin2lin, METH_OLDARGS },
+	{ "adpcm2lin", audioop_adpcm2lin, METH_OLDARGS },
+	{ "lin2adpcm", audioop_lin2adpcm, METH_OLDARGS },
+	{ "tomono", audioop_tomono, METH_OLDARGS },
+	{ "tostereo", audioop_tostereo, METH_OLDARGS },
+	{ "getsample", audioop_getsample, METH_OLDARGS },
+	{ "reverse", audioop_reverse, METH_OLDARGS },
+	{ "ratecv", audioop_ratecv, METH_VARARGS },
 	{ 0,          0 }
 };
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initaudioop(void)
 {
 	PyObject *m, *d;
