@@ -179,7 +179,7 @@ static unsigned short crctab_hqx[256] = {
 	0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0,
 };
 
-static char doc_a2b_uu[] = "(ascii) -> bin. Decode a line of uuencoded data";
+PyDoc_STRVAR(doc_a2b_uu, "(ascii) -> bin. Decode a line of uuencoded data");
 
 static PyObject *
 binascii_a2b_uu(PyObject *self, PyObject *args)
@@ -204,7 +204,8 @@ binascii_a2b_uu(PyObject *self, PyObject *args)
 	bin_data = (unsigned char *)PyString_AsString(rv);
 
 	for( ; bin_len > 0 ; ascii_len--, ascii_data++ ) {
-		this_ch = *ascii_data;
+		/* XXX is it really best to add NULs if there's no more data */
+		this_ch = (ascii_len > 0) ? *ascii_data : 0;
 		if ( this_ch == '\n' || this_ch == '\r' || ascii_len <= 0) {
 			/*
 			** Whitespace. Assume some spaces got eaten at
@@ -254,7 +255,7 @@ binascii_a2b_uu(PyObject *self, PyObject *args)
 	return rv;
 }
 
-static char doc_b2a_uu[] = "(bin) -> ascii. Uuencode line of data";
+PyDoc_STRVAR(doc_b2a_uu, "(bin) -> ascii. Uuencode line of data");
 
 static PyObject *
 binascii_b2a_uu(PyObject *self, PyObject *args)
@@ -275,7 +276,7 @@ binascii_b2a_uu(PyObject *self, PyObject *args)
 	}
 
 	/* We're lazy and allocate to much (fixed up later) */
-	if ( (rv=PyString_FromStringAndSize(NULL, bin_len*2)) == NULL )
+	if ( (rv=PyString_FromStringAndSize(NULL, bin_len*2+2)) == NULL )
 		return NULL;
 	ascii_data = (unsigned char *)PyString_AsString(rv);
 
@@ -330,7 +331,7 @@ binascii_find_valid(unsigned char *s, int slen, int num)
 	return ret;
 }
 
-static char doc_a2b_base64[] = "(ascii) -> bin. Decode a line of base64 data";
+PyDoc_STRVAR(doc_a2b_base64, "(ascii) -> bin. Decode a line of base64 data");
 
 static PyObject *
 binascii_a2b_base64(PyObject *self, PyObject *args)
@@ -408,17 +409,20 @@ binascii_a2b_base64(PyObject *self, PyObject *args)
 		return NULL;
 	}
 
-	/* and set string size correctly */
+	/* And set string size correctly. If the result string is empty
+	** (because the input was all invalid) return the shared empty
+	** string instead; _PyString_Resize() won't do this for us.
+	*/
 	if (bin_len > 0)
 		_PyString_Resize(&rv, bin_len);
 	else {
 		Py_DECREF(rv);
-		return PyString_FromString("");
+		rv = PyString_FromString("");
 	}
 	return rv;
 }
 
-static char doc_b2a_base64[] = "(bin) -> ascii. Base64-code line of data";
+PyDoc_STRVAR(doc_b2a_base64, "(bin) -> ascii. Base64-code line of data");
 
 static PyObject *
 binascii_b2a_base64(PyObject *self, PyObject *args)
@@ -471,7 +475,7 @@ binascii_b2a_base64(PyObject *self, PyObject *args)
 	return rv;
 }
 
-static char doc_a2b_hqx[] = "ascii -> bin, done. Decode .hqx coding";
+PyDoc_STRVAR(doc_a2b_hqx, "ascii -> bin, done. Decode .hqx coding");
 
 static PyObject *
 binascii_a2b_hqx(PyObject *self, PyObject *args)
@@ -487,8 +491,10 @@ binascii_a2b_hqx(PyObject *self, PyObject *args)
 	if ( !PyArg_ParseTuple(args, "t#:a2b_hqx", &ascii_data, &len) )
 		return NULL;
 
-	/* Allocate a string that is too big (fixed later) */
-	if ( (rv=PyString_FromStringAndSize(NULL, len)) == NULL )
+	/* Allocate a string that is too big (fixed later) 
+	   Add two to the initial length to prevent interning which
+	   would preclude subsequent resizing.  */
+	if ( (rv=PyString_FromStringAndSize(NULL, len+2)) == NULL )
 		return NULL;
 	bin_data = (unsigned char *)PyString_AsString(rv);
 
@@ -535,7 +541,7 @@ binascii_a2b_hqx(PyObject *self, PyObject *args)
 	return NULL;
 }
 
-static char doc_rlecode_hqx[] = "Binhex RLE-code binary data";
+PyDoc_STRVAR(doc_rlecode_hqx, "Binhex RLE-code binary data");
 
 static PyObject *
 binascii_rlecode_hqx(PyObject *self, PyObject *args)
@@ -549,7 +555,7 @@ binascii_rlecode_hqx(PyObject *self, PyObject *args)
 		return NULL;
 
 	/* Worst case: output is twice as big as input (fixed later) */
-	if ( (rv=PyString_FromStringAndSize(NULL, len*2)) == NULL )
+	if ( (rv=PyString_FromStringAndSize(NULL, len*2+2)) == NULL )
 		return NULL;
 	out_data = (unsigned char *)PyString_AsString(rv);
 
@@ -582,7 +588,7 @@ binascii_rlecode_hqx(PyObject *self, PyObject *args)
 	return rv;
 }
 
-static char doc_b2a_hqx[] = "Encode .hqx data";
+PyDoc_STRVAR(doc_b2a_hqx, "Encode .hqx data");
 
 static PyObject *
 binascii_b2a_hqx(PyObject *self, PyObject *args)
@@ -598,7 +604,7 @@ binascii_b2a_hqx(PyObject *self, PyObject *args)
 		return NULL;
 
 	/* Allocate a buffer that is at least large enough */
-	if ( (rv=PyString_FromStringAndSize(NULL, len*2)) == NULL )
+	if ( (rv=PyString_FromStringAndSize(NULL, len*2+2)) == NULL )
 		return NULL;
 	ascii_data = (unsigned char *)PyString_AsString(rv);
 
@@ -622,7 +628,7 @@ binascii_b2a_hqx(PyObject *self, PyObject *args)
 	return rv;
 }
 
-static char doc_rledecode_hqx[] = "Decode hexbin RLE-coded string";
+PyDoc_STRVAR(doc_rledecode_hqx, "Decode hexbin RLE-coded string");
 
 static PyObject *
 binascii_rledecode_hqx(PyObject *self, PyObject *args)
@@ -718,8 +724,8 @@ binascii_rledecode_hqx(PyObject *self, PyObject *args)
 	return rv;
 }
 
-static char doc_crc_hqx[] =
-"(data, oldcrc) -> newcrc. Compute hqx CRC incrementally";
+PyDoc_STRVAR(doc_crc_hqx,
+"(data, oldcrc) -> newcrc. Compute hqx CRC incrementally");
 
 static PyObject *
 binascii_crc_hqx(PyObject *self, PyObject *args)
@@ -738,8 +744,8 @@ binascii_crc_hqx(PyObject *self, PyObject *args)
 	return Py_BuildValue("i", crc);
 }
 
-static char doc_crc32[] =
-"(data, oldcrc = 0) -> newcrc. Compute CRC-32 incrementally";
+PyDoc_STRVAR(doc_crc32,
+"(data, oldcrc = 0) -> newcrc. Compute CRC-32 incrementally");
 
 /*  Crc - 32 BIT ANSI X3.66 CRC checksum files
     Also known as: ISO 3307
@@ -929,10 +935,10 @@ binascii_hexlify(PyObject *self, PyObject *args)
 	return NULL;
 }
 
-static char doc_hexlify[] =
+PyDoc_STRVAR(doc_hexlify,
 "b2a_hex(data) -> s; Hexadecimal representation of binary data.\n\
 \n\
-This function is also available as \"hexlify()\".";
+This function is also available as \"hexlify()\".");
 
 
 static int
@@ -995,11 +1001,11 @@ binascii_unhexlify(PyObject *self, PyObject *args)
 	return NULL;
 }
 
-static char doc_unhexlify[] =
+PyDoc_STRVAR(doc_unhexlify,
 "a2b_hex(hexstr) -> s; Binary data of hexadecimal representation.\n\
 \n\
 hexstr must contain an even number of hex digits (upper or lower case).\n\
-This function is also available as \"unhexlify()\"";
+This function is also available as \"unhexlify()\"");
 
 static int table_hex[128] = {
   -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
@@ -1016,7 +1022,7 @@ static int table_hex[128] = {
 
 #define MAXLINESIZE 76
 
-static char doc_a2b_qp[] = "Decode a string of qp-encoded data";
+PyDoc_STRVAR(doc_a2b_qp, "Decode a string of qp-encoded data");
 
 static PyObject*
 binascii_a2b_qp(PyObject *self, PyObject *args, PyObject *kwargs)
@@ -1033,13 +1039,16 @@ binascii_a2b_qp(PyObject *self, PyObject *args, PyObject *kwargs)
 	      &datalen, &header))
 		return NULL;
 
-	/* We allocate the output same size as input, this is overkill */
-	odata = (unsigned char *) calloc(1, datalen);
-
+	/* We allocate the output same size as input, this is overkill.
+	 * The previous implementation used calloc() so we'll zero out the
+	 * memory here too, since PyMem_Malloc() does not guarantee that.
+	 */
+	odata = (unsigned char *) PyMem_Malloc(datalen);
 	if (odata == NULL) {
 		PyErr_NoMemory();
 		return NULL;
 	}
+	memset(odata, 0, datalen);
 
 	in = out = 0;
 	while (in < datalen) {
@@ -1087,10 +1096,10 @@ binascii_a2b_qp(PyObject *self, PyObject *args, PyObject *kwargs)
 		}
 	}
 	if ((rv = PyString_FromStringAndSize((char *)odata, out)) == NULL) {
-		free (odata);
+		PyMem_Free(odata);
 		return NULL;
 	}
-	free (odata);
+	PyMem_Free(odata);
 	return rv;
 }
 
@@ -1105,13 +1114,13 @@ to_hex (unsigned char ch, unsigned char *s)
 	return 0;
 }
 
-static char doc_b2a_qp[] = 
+PyDoc_STRVAR(doc_b2a_qp,
 "b2a_qp(data, quotetabs=0, istext=1, header=0) -> s; \n\
  Encode a string using quoted-printable encoding. \n\
 \n\
 On encoding, when istext is set, newlines are not encoded, and white \n\
 space at end of lines is.  When istext is not set, \\r and \\n (CR/LF) are \n\
-both encoded.  When quotetabs is set, space and tabs are encoded.";
+both encoded.  When quotetabs is set, space and tabs are encoded.");
 
 /* XXX: This is ridiculously complicated to be backward compatible
  * (mostly) with the quopri module.  It doesn't re-create the quopri
@@ -1204,12 +1213,16 @@ binascii_b2a_qp (PyObject *self, PyObject *args, PyObject *kwargs)
 		}
 	}
 
-	odata = (unsigned char *) calloc(1, odatalen);
-
+	/* We allocate the output same size as input, this is overkill.
+	 * The previous implementation used calloc() so we'll zero out the
+	 * memory here too, since PyMem_Malloc() does not guarantee that.
+	 */
+	odata = (unsigned char *) PyMem_Malloc(odatalen);
 	if (odata == NULL) {
 		PyErr_NoMemory();
 		return NULL;
 	}
+	memset(odata, 0, odatalen);
 
 	in = out = linelen = 0;
 	while (in < datalen) {
@@ -1278,10 +1291,10 @@ binascii_b2a_qp (PyObject *self, PyObject *args, PyObject *kwargs)
 		}
 	}
 	if ((rv = PyString_FromStringAndSize((char *)odata, out)) == NULL) {
-		free (odata);
+		PyMem_Free(odata);
 		return NULL;
 	}
-	free (odata);
+	PyMem_Free(odata);
 	return rv;
 }
 
@@ -1312,9 +1325,9 @@ static struct PyMethodDef binascii_module_methods[] = {
 
 
 /* Initialization function for the module (*must* be called initbinascii) */
-static char doc_binascii[] = "Conversion between binary data and ASCII";
+PyDoc_STRVAR(doc_binascii, "Conversion between binary data and ASCII");
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initbinascii(void)
 {
 	PyObject *m, *d, *x;
