@@ -22,6 +22,18 @@ static char __author__[] =
     Gustavo Niemeyer <niemeyer@conectiva.com>\n\
 ";
 
+/* Our very own off_t-like type, 64-bit if possible */
+/* copied from Objects/fileobject.c */
+#if !defined(HAVE_LARGEFILE_SUPPORT)
+typedef off_t Py_off_t;
+#elif SIZEOF_OFF_T >= 8
+typedef off_t Py_off_t;
+#elif SIZEOF_FPOS_T >= 8
+typedef fpos_t Py_off_t;
+#else
+#error "Large file support, but neither off_t nor fpos_t is large enough."
+#endif
+
 #define BUF(v) PyString_AS_STRING((PyStringObject *)v)
 
 #define MODE_CLOSED   0
@@ -42,7 +54,7 @@ static char __author__[] =
 	(((PY_LONG_LONG)bzs->total_out_hi32 << 32) + bzs->total_out_lo32)
 #else
 #define BZS_TOTAL_OUT(bzs) \
-	bzs->total_out_lo32;
+	bzs->total_out_lo32
 #endif
 
 #else /* ! BZ_CONFIG_ERROR */
@@ -98,8 +110,8 @@ typedef struct {
 
 	BZFILE *fp;
 	int mode;
-	long pos;
-	long size;
+	Py_off_t pos;
+	Py_off_t size;
 #ifdef WITH_THREAD
 	PyThread_type_lock lock;
 #endif
@@ -405,7 +417,9 @@ Util_ReadAhead(BZ2FileObject *f, int bufsize)
 			Util_DropReadAhead(f);
 	}
 	if (f->mode == MODE_READ_EOF) {
-		return -1;
+		f->f_bufptr = f->f_buf;
+		f->f_bufend = f->f_buf;
+		return 0;
 	}
 	if ((f->f_buf = PyMem_Malloc(bufsize)) == NULL) {
 		return -1;
@@ -682,13 +696,13 @@ BZ2File_readlines(BZ2FileObject *self, PyObject *args)
 		}
 		totalread += nread;
 		p = memchr(buffer+nfilled, '\n', nread);
-		if (p == NULL) {
+		if (!shortread && p == NULL) {
 			/* Need a larger buffer to fit this line */
 			nfilled += nread;
 			buffersize *= 2;
 			if (buffersize > INT_MAX) {
 				PyErr_SetString(PyExc_OverflowError,
-			    "line is longer than a Python string can hold");
+				"line is longer than a Python string can hold");
 				goto error;
 			}
 			if (big_buffer == NULL) {
@@ -709,7 +723,7 @@ BZ2File_readlines(BZ2FileObject *self, PyObject *args)
 		}
 		end = buffer+nfilled+nread;
 		q = buffer;
-		do {
+		while (p != NULL) {
 			/* Process complete lines */
 			p++;
 			line = PyString_FromStringAndSize(q, p-q);
@@ -721,7 +735,7 @@ BZ2File_readlines(BZ2FileObject *self, PyObject *args)
 				goto error;
 			q = p;
 			p = memchr(q, '\n', end-q);
-		} while (p != NULL);
+		}
 		/* Move the remaining incomplete line to the start */
 		nfilled = end-q;
 		memmove(buffer, q, nfilled);
@@ -962,18 +976,27 @@ static PyObject *
 BZ2File_seek(BZ2FileObject *self, PyObject *args)
 {
 	int where = 0;
-	long offset;
+	PyObject *offobj;
+	Py_off_t offset;
 	char small_buffer[SMALLCHUNK];
 	char *buffer = small_buffer;
 	size_t buffersize = SMALLCHUNK;
 	int bytesread = 0;
-	int readsize;
+	size_t readsize;
 	int chunksize;
 	int bzerror;
 	int rewind = 0;
 	PyObject *ret = NULL;
 
-	if (!PyArg_ParseTuple(args, "l|i:seek", &offset, &where))
+	if (!PyArg_ParseTuple(args, "O|i:seek", &offobj, &where))
+		return NULL;
+#if !defined(HAVE_LARGEFILE_SUPPORT)
+	offset = PyInt_AsLong(offobj);
+#else
+	offset = PyLong_Check(offobj) ?
+		PyLong_AsLongLong(offobj) : PyInt_AsLong(offobj);
+#endif
+	if (PyErr_Occurred())
 		return NULL;
 
 	ACQUIRE_LOCK(self);
@@ -994,50 +1017,43 @@ BZ2File_seek(BZ2FileObject *self, PyObject *args)
 			goto cleanup;;
 	}
 
-	if (offset < 0) {
-		if (where == 1) {
-			offset = self->pos + offset;
-			rewind = 1;
-		} else if (where == 2) {
-			if (self->size == -1) {
-				assert(self->mode != MODE_READ_EOF);
-				for (;;) {
-					Py_BEGIN_ALLOW_THREADS
-					chunksize = Util_UnivNewlineRead(
-							&bzerror, self->fp,
-							buffer, buffersize,
-							self);
-					self->pos += chunksize;
-					Py_END_ALLOW_THREADS
+	if (where == 2) {
+		if (self->size == -1) {
+			assert(self->mode != MODE_READ_EOF);
+			for (;;) {
+				Py_BEGIN_ALLOW_THREADS
+				chunksize = Util_UnivNewlineRead(
+						&bzerror, self->fp,
+						buffer, buffersize,
+						self);
+				self->pos += chunksize;
+				Py_END_ALLOW_THREADS
 
-					bytesread += chunksize;
-					if (bzerror == BZ_STREAM_END) {
-						break;
-					} else if (bzerror != BZ_OK) {
-						Util_CatchBZ2Error(bzerror);
-						goto cleanup;
-					}
+				bytesread += chunksize;
+				if (bzerror == BZ_STREAM_END) {
+					break;
+				} else if (bzerror != BZ_OK) {
+					Util_CatchBZ2Error(bzerror);
+					goto cleanup;
 				}
-				self->mode = MODE_READ_EOF;
-				self->size = self->pos;
-				bytesread = 0;
 			}
-			offset = self->size + offset;
-			if (offset >= self->pos)
-				offset -= self->pos;
-			else
-				rewind = 1;
+			self->mode = MODE_READ_EOF;
+			self->size = self->pos;
+			bytesread = 0;
 		}
-		if (offset < 0)
-			offset = 0;
-	} else if (where == 0) {
-		if (offset >= self->pos)
-			offset -= self->pos;
-		else
-			rewind = 1;
+		offset = self->size + offset;
+	} else if (where == 1) {
+		offset = self->pos + offset;
 	}
 
-	if (rewind) {
+	/* Before getting here, offset must be the absolute position the file 
+	 * pointer should be set to. */
+
+	if (offset >= self->pos) {
+		/* we can move forward */
+		offset -= self->pos;
+	} else {
+		/* we cannot move back, so rewind the stream */
 		BZ2_bzReadClose(&bzerror, self->fp);
 		if (bzerror != BZ_OK) {
 			Util_CatchBZ2Error(bzerror);
@@ -1056,20 +1072,21 @@ BZ2File_seek(BZ2FileObject *self, PyObject *args)
 			goto cleanup;
 		}
 		self->mode = MODE_READ;
-	} else if (self->mode == MODE_READ_EOF) {
-		goto exit;
 	}
 
-	if (offset == 0)
+	if (offset <= 0 || self->mode == MODE_READ_EOF)
 		goto exit;
 
 	/* Before getting here, offset must be set to the number of bytes
 	 * to walk forward. */
 	for (;;) {
-		if ((size_t)offset-bytesread > buffersize)
+		if (offset-bytesread > buffersize)
 			readsize = buffersize;
 		else
-			readsize = offset-bytesread;
+			/* offset might be wider that readsize, but the result
+			 * of the subtraction is bound by buffersize (see the
+			 * condition above). buffersize is 8192. */
+			readsize = (size_t)(offset-bytesread);
 		Py_BEGIN_ALLOW_THREADS
 		chunksize = Util_UnivNewlineRead(&bzerror, self->fp,
 						 buffer, readsize, self);
@@ -1114,7 +1131,11 @@ BZ2File_tell(BZ2FileObject *self, PyObject *args)
 		goto cleanup;
 	}
 
+#if !defined(HAVE_LARGEFILE_SUPPORT)
 	ret = PyInt_FromLong(self->pos);
+#else
+	ret = PyLong_FromLongLong(self->pos);
+#endif
 
 cleanup:
 	return ret;
@@ -1308,6 +1329,10 @@ BZ2File_init(BZ2FileObject *self, PyObject *args, PyObject *kwargs)
 			break;
 	}
 
+	if (mode_char == 0) {
+		mode_char = 'r';
+	}
+
 	mode = (mode_char == 'r') ? "rb" : "wb";
 
 	self->file = PyObject_CallFunction((PyObject*)&PyFile_Type, "(Osi)",
