@@ -1,7 +1,7 @@
 /*
  /  Author: Sam Rushing <rushing@nightmare.com>
  /  Hacked for Unix by AMK
- /  $Id: mmapmodule.c,v 2.48 2004/05/19 14:39:08 akuchling Exp $
+ /  $Id: mmapmodule.c 42245 2006-02-05 06:00:54Z neal.norwitz $
 
  / mmapmodule.cpp -- map a view of a file into memory
  /
@@ -99,6 +99,8 @@ mmap_object_dealloc(mmap_object *m_obj)
 #endif /* MS_WINDOWS */
 
 #ifdef UNIX
+	if (m_obj->fd >= 0)
+		(void) close(m_obj->fd);
 	if (m_obj->data!=NULL) {
 		msync(m_obj->data, m_obj->size, MS_SYNC);
 		munmap(m_obj->data, m_obj->size);
@@ -136,6 +138,8 @@ mmap_close_method(mmap_object *self, PyObject *args)
 #endif /* MS_WINDOWS */
 
 #ifdef UNIX
+	(void) close(self->fd);
+	self->fd = -1;
 	if (self->data != NULL) {
 		munmap(self->data, self->size);
 		self->data = NULL;
@@ -423,6 +427,11 @@ mmap_resize_method(mmap_object *self,
 	} else {
 		void *newmap;
 
+		if (ftruncate(self->fd, new_size) == -1) {
+			PyErr_SetFromErrno(mmap_module_error);
+			return NULL;
+		}
+
 #ifdef MREMAP_MAYMOVE
 		newmap = mremap(self->data, self->size, new_size, MREMAP_MAYMOVE);
 #else
@@ -905,9 +914,20 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 #endif
 	m_obj = PyObject_New (mmap_object, &mmap_object_type);
 	if (m_obj == NULL) {return NULL;}
+	m_obj->data = NULL;
 	m_obj->size = (size_t) map_size;
 	m_obj->pos = (size_t) 0;
-	m_obj->fd = fd;
+	if (fd == -1) {
+		m_obj->fd = -1;
+	} else {
+		m_obj->fd = dup(fd);
+		if (m_obj->fd == -1) {
+			Py_DECREF(m_obj);
+			PyErr_SetFromErrno(mmap_module_error);
+			return NULL;
+		}
+	}
+
 	m_obj->data = mmap(NULL, map_size, 
 			   prot, flags,
 			   fd, 0);
