@@ -246,9 +246,9 @@ regobj_group(regexobject *re, PyObject *args)
 
 
 static struct PyMethodDef reg_methods[] = {
-	{"match",	(PyCFunction)regobj_match, 1},
-	{"search",	(PyCFunction)regobj_search, 1},
-	{"group",	(PyCFunction)regobj_group, 1},
+	{"match",	(PyCFunction)regobj_match, METH_VARARGS},
+	{"search",	(PyCFunction)regobj_search, METH_VARARGS},
+	{"group",	(PyCFunction)regobj_group, METH_VARARGS},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -516,11 +516,8 @@ symcomp(PyObject *pattern, PyObject *gdict)
 		return NULL;
 	}
 	/* _PyString_Resize() decrements npattern on failure */
-	if (_PyString_Resize(&npattern, n - v) == 0)
-		return npattern;
-	else {
-		return NULL;
-	}
+	_PyString_Resize(&npattern, n - v);
+	return npattern;
 
 }
 
@@ -538,8 +535,7 @@ regex_symcomp(PyObject *self, PyObject *args)
 
 	gdict = PyDict_New();
 	if (gdict == NULL || (npattern = symcomp(pattern, gdict)) == NULL) {
-		Py_DECREF(gdict);
-		Py_DECREF(pattern);
+		Py_XDECREF(gdict);
 		return NULL;
 	}
 	retval = newregexobject(npattern, tran, pattern, gdict);
@@ -554,7 +550,7 @@ static PyObject *cache_prog;
 static int
 update_cache(PyObject *pat)
 {
-	PyObject *tuple = Py_BuildValue("(O)", pat);
+	PyObject *tuple = PyTuple_Pack(1, pat);
 	int status = 0;
 
 	if (!tuple)
@@ -583,7 +579,7 @@ regex_match(PyObject *self, PyObject *args)
 	PyObject *pat, *string;
 	PyObject *tuple, *v;
 
-	if (!PyArg_Parse(args, "(SS)", &pat, &string))
+	if (!PyArg_ParseTuple(args, "SS:match", &pat, &string))
 		return NULL;
 	if (update_cache(pat) < 0)
 		return NULL;
@@ -601,7 +597,7 @@ regex_search(PyObject *self, PyObject *args)
 	PyObject *pat, *string;
 	PyObject *tuple, *v;
 
-	if (!PyArg_Parse(args, "(SS)", &pat, &string))
+	if (!PyArg_ParseTuple(args, "SS:search", &pat, &string))
 		return NULL;
 	if (update_cache(pat) < 0)
 		return NULL;
@@ -617,7 +613,7 @@ static PyObject *
 regex_set_syntax(PyObject *self, PyObject *args)
 {
 	int syntax;
-	if (!PyArg_Parse(args, "i", &syntax))
+	if (!PyArg_ParseTuple(args, "i:set_syntax", &syntax))
 		return NULL;
 	syntax = re_set_syntax(syntax);
 	/* wipe the global pattern cache */
@@ -629,25 +625,23 @@ regex_set_syntax(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-regex_get_syntax(PyObject *self, PyObject *args)
+regex_get_syntax(PyObject *self)
 {
-	if (!PyArg_Parse(args, ""))
-		return NULL;
 	return PyInt_FromLong((long)re_syntax);
 }
 
 
 static struct PyMethodDef regex_global_methods[] = {
-	{"compile",	regex_compile, 1},
-	{"symcomp",	regex_symcomp, 1},
-	{"match",	regex_match, 0},
-	{"search",	regex_search, 0},
-	{"set_syntax",	regex_set_syntax, 0},
-	{"get_syntax",  regex_get_syntax, 0},
+	{"compile",	regex_compile, METH_VARARGS},
+	{"symcomp",	regex_symcomp, METH_VARARGS},
+	{"match",	regex_match, METH_VARARGS},
+	{"search",	regex_search, METH_VARARGS},
+	{"set_syntax",	regex_set_syntax, METH_VARARGS},
+	{"get_syntax",  (PyCFunction)regex_get_syntax, METH_NOARGS},
 	{NULL,		NULL}		     /* sentinel */
 };
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initregex(void)
 {
 	PyObject *m, *d, *v;
