@@ -65,6 +65,9 @@ zipimporter_init(ZipImporter *self, PyObject *args, PyObject *kwds)
 	char *path, *p, *prefix, buf[MAXPATHLEN+2];
 	int len;
 
+	if (!_PyArg_NoKeywords("zipimporter()", kwds))
+		return -1;
+
 	if (!PyArg_ParseTuple(args, "s:zipimporter",
 			      &path))
 		return -1;
@@ -431,8 +434,7 @@ zipimporter_get_data(PyObject *obj, PyObject *args)
 
 	toc_entry = PyDict_GetItemString(self->files, path);
 	if (toc_entry == NULL) {
-		PyErr_Format(PyExc_IOError, "file not found [%.200s]",
-			     path);
+		PyErr_SetFromErrnoWithFilename(PyExc_IOError, path);
 		return NULL;
 	}
 	return get_data(PyString_AsString(self->archive), toc_entry);
@@ -637,7 +639,8 @@ get_long(unsigned char *buf) {
 
    A toc_entry is a tuple:
 
-       (compress,      # compression kind; 0 for uncompressed
+       (__file__,      # value to use for __file__, available for all files
+        compress,      # compression kind; 0 for uncompressed
         data_size,     # size of compressed data on disk
         file_size,     # size of decompressed data
         file_offset,   # offset of file header from start of archive
@@ -1162,6 +1165,8 @@ initzipimport(void)
 
 	mod = Py_InitModule4("zipimport", NULL, zipimport_doc,
 			     NULL, PYTHON_API_VERSION);
+	if (mod == NULL)
+		return;
 
 	ZipImportError = PyErr_NewException("zipimport.ZipImportError",
 					    PyExc_ImportError, NULL);
