@@ -3,25 +3,54 @@
 
 #include "Python.h"
 
-#ifdef macintosh
-#include "macglue.h"
+#ifdef Py_REF_DEBUG
+long _Py_RefTotal;
 #endif
 
-/* just for trashcan: */
-#include "compile.h"
-#include "frameobject.h"
-#include "traceback.h"
-
-#if defined( Py_TRACE_REFS ) || defined( Py_REF_DEBUG )
-DL_IMPORT(long) _Py_RefTotal;
-#endif
-
-DL_IMPORT(int) Py_DivisionWarningFlag;
+int Py_DivisionWarningFlag;
 
 /* Object allocation routines used by NEWOBJ and NEWVAROBJ macros.
    These are used by the individual routines for object creation.
    Do not call them otherwise, they do not initialize the object! */
 
+#ifdef Py_TRACE_REFS
+/* Head of circular doubly-linked list of all objects.  These are linked
+ * together via the _ob_prev and _ob_next members of a PyObject, which
+ * exist only in a Py_TRACE_REFS build.
+ */
+static PyObject refchain = {&refchain, &refchain};
+
+/* Insert op at the front of the list of all objects.  If force is true,
+ * op is added even if _ob_prev and _ob_next are non-NULL already.  If
+ * force is false amd _ob_prev or _ob_next are non-NULL, do nothing.
+ * force should be true if and only if op points to freshly allocated,
+ * uninitialized memory, or you've unlinked op from the list and are
+ * relinking it into the front.
+ * Note that objects are normally added to the list via _Py_NewReference,
+ * which is called by PyObject_Init.  Not all objects are initialized that
+ * way, though; exceptions include statically allocated type objects, and
+ * statically allocated singletons (like Py_True and Py_None).
+ */
+void
+_Py_AddToAllObjects(PyObject *op, int force)
+{
+#ifdef  Py_DEBUG
+	if (!force) {
+		/* If it's initialized memory, op must be in or out of
+		 * the list unambiguously.
+		 */
+		assert((op->_ob_prev == NULL) == (op->_ob_next == NULL));
+	}
+#endif
+	if (force || op->_ob_prev == NULL) {
+		op->_ob_next = refchain._ob_next;
+		op->_ob_prev = &refchain;
+		refchain._ob_next->_ob_prev = op;
+		refchain._ob_next = op;
+	}
+}
+#endif	/* Py_TRACE_REFS */
+
 #ifdef COUNT_ALLOCS
 static PyTypeObject *type_list;
 extern int tuple_zero_allocs, fast_tuple_allocs;
@@ -79,7 +108,22 @@ inc_count(PyTypeObject *tp)
 		if (tp->tp_next != NULL) /* sanity check */
 			Py_FatalError("XXX inc_count sanity check");
 		tp->tp_next = type_list;
+		/* Note that as of Python 2.2, heap-allocated type objects
+		 * can go away, but this code requires that they stay alive
+		 * until program exit.  That's why we're careful with
+		 * refcounts here.  type_list gets a new reference to tp,
+		 * while ownership of the reference type_list used to hold
+		 * (if any) was transferred to tp->tp_next in the line above.
+		 * tp is thus effectively immortal after this.
+		 */
+		Py_INCREF(tp);
 		type_list = tp;
+#ifdef Py_TRACE_REFS
+		/* Also insert in the doubly-linked list of all objects,
+		 * if not already there.
+		 */
+		_Py_AddToAllObjects((PyObject *)tp, 0);
+#endif
 	}
 	tp->tp_allocs++;
 	if (tp->tp_allocs - tp->tp_frees > tp->tp_maxalloc)
@@ -87,14 +131,38 @@ inc_count(PyTypeObject *tp)
 }
 #endif
 
+#ifdef Py_REF_DEBUG
+/* Log a fatal error; doesn't return. */
+void
+_Py_NegativeRefcount(const char *fname, int lineno, PyObject *op)
+{
+	char buf[300];
+
+	PyOS_snprintf(buf, sizeof(buf),
+		      "%s:%i object at %p has negative ref count %i",
+		      fname, lineno, op, op->ob_refcnt);
+	Py_FatalError(buf);
+}
+
+#endif /* Py_REF_DEBUG */
+
+void
+Py_IncRef(PyObject *o)
+{
+    Py_XINCREF(o);
+}
+
+void
+Py_DecRef(PyObject *o)
+{
+    Py_XDECREF(o);
+}
+
 PyObject *
 PyObject_Init(PyObject *op, PyTypeObject *tp)
 {
-	if (op == NULL) {
-		PyErr_SetString(PyExc_SystemError,
-				"NULL object passed to PyObject_Init");
-		return op;
-  	}
+	if (op == NULL)
+		return PyErr_NoMemory();
 	/* Any changes should be reflected in PyObject_INIT (objimpl.h) */
 	op->ob_type = tp;
 	_Py_NewReference(op);
@@ -104,11 +172,8 @@ PyObject_Init(PyObject *op, PyTypeObject *tp)
 PyVarObject *
 PyObject_InitVar(PyVarObject *op, PyTypeObject *tp, int size)
 {
-	if (op == NULL) {
-		PyErr_SetString(PyExc_SystemError,
-				"NULL object passed to PyObject_InitVar");
-		return op;
-	}
+	if (op == NULL)
+		return (PyVarObject *) PyErr_NoMemory();
 	/* Any changes should be reflected in PyObject_INIT_VAR */
 	op->ob_size = size;
 	op->ob_type = tp;
@@ -137,16 +202,23 @@ _PyObject_NewVar(PyTypeObject *tp, int nitems)
 	return PyObject_INIT_VAR(op, tp, nitems);
 }
 
+/* for binary compatibility with 2.2 */
+#undef _PyObject_Del
 void
 _PyObject_Del(PyObject *op)
 {
 	PyObject_FREE(op);
 }
 
-int
-PyObject_Print(PyObject *op, FILE *fp, int flags)
+/* Implementation of PyObject_Print with recursion checking */
+static int
+internal_print(PyObject *op, FILE *fp, int flags, int nesting)
 {
 	int ret = 0;
+	if (nesting > 10) {
+		PyErr_SetString(PyExc_RuntimeError, "print recursion");
+		return -1;
+	}
 	if (PyErr_CheckSignals())
 		return -1;
 #ifdef USE_STACKCHECK
@@ -172,7 +244,8 @@ PyObject_Print(PyObject *op, FILE *fp, int flags)
 			if (s == NULL)
 				ret = -1;
 			else {
-				ret = PyObject_Print(s, fp, Py_PRINT_RAW);
+				ret = internal_print(s, fp, Py_PRINT_RAW,
+						     nesting+1);
 			}
 			Py_XDECREF(s);
 		}
@@ -189,8 +262,15 @@ PyObject_Print(PyObject *op, FILE *fp, int flags)
 	return ret;
 }
 
+int
+PyObject_Print(PyObject *op, FILE *fp, int flags)
+{
+	return internal_print(op, fp, flags, 0);
+}
+
+
 /* For debugging convenience.  See Misc/gdbinit for some useful gdb hooks */
-void _PyObject_Dump(PyObject* op) 
+void _PyObject_Dump(PyObject* op)
 {
 	if (op == NULL)
 		fprintf(stderr, "NULL\n");
@@ -254,7 +334,7 @@ PyObject *
 PyObject_Str(PyObject *v)
 {
 	PyObject *res;
-	
+
 	if (v == NULL)
 		return PyString_FromString("<NULL>");
 	if (PyString_CheckExact(v)) {
@@ -293,10 +373,16 @@ PyObject *
 PyObject_Unicode(PyObject *v)
 {
 	PyObject *res;
-	
-	if (v == NULL)
+	PyObject *str;
+
+	if (v == NULL) {
 		res = PyString_FromString("<NULL>");
-	if (PyUnicode_CheckExact(v)) {
+		if (res == NULL)
+			return NULL;
+		str = PyUnicode_FromEncodedObject(res, NULL, "strict");
+		Py_DECREF(res);
+		return str;
+	} else if (PyUnicode_CheckExact(v)) {
 		Py_INCREF(v);
 		return v;
 	}
@@ -338,25 +424,61 @@ PyObject_Unicode(PyObject *v)
 	if (res == NULL)
 		return NULL;
 	if (!PyUnicode_Check(res)) {
-		PyObject *str;
 		str = PyUnicode_FromEncodedObject(res, NULL, "strict");
 		Py_DECREF(res);
-		if (str)
-			res = str;
-		else
-		    	return NULL;
+		res = str;
 	}
 	return res;
 }
 #endif
 
 
+/* Helper to warn about deprecated tp_compare return values.  Return:
+   -2 for an exception;
+   -1 if v <  w;
+    0 if v == w;
+    1 if v  > w.
+   (This function cannot return 2.)
+*/
+static int
+adjust_tp_compare(int c)
+{
+	if (PyErr_Occurred()) {
+		if (c != -1 && c != -2) {
+			PyObject *t, *v, *tb;
+			PyErr_Fetch(&t, &v, &tb);
+			if (PyErr_Warn(PyExc_RuntimeWarning,
+				       "tp_compare didn't return -1 or -2 "
+				       "for exception") < 0) {
+				Py_XDECREF(t);
+				Py_XDECREF(v);
+				Py_XDECREF(tb);
+			}
+			else
+				PyErr_Restore(t, v, tb);
+		}
+		return -2;
+	}
+	else if (c < -1 || c > 1) {
+		if (PyErr_Warn(PyExc_RuntimeWarning,
+			       "tp_compare didn't return -1, 0 or 1") < 0)
+			return -2;
+		else
+			return c < -1 ? -1 : 1;
+	}
+	else {
+		assert(c >= -1 && c <= 1);
+		return c;
+	}
+}
+
+
 /* Macro to get the tp_richcompare field of a type if defined */
 #define RICHCOMPARE(t) (PyType_HasFeature((t), Py_TPFLAGS_HAVE_RICHCOMPARE) \
                          ? (t)->tp_richcompare : NULL)
 
 /* Map rich comparison operators to their swapped version, e.g. LT --> GT */
-static int swapped_op[] = {Py_GT, Py_GE, Py_EQ, Py_NE, Py_LT, Py_LE};
+int _Py_SwappedOp[] = {Py_GT, Py_GE, Py_EQ, Py_NE, Py_LT, Py_LE};
 
 /* Try a genuine rich comparison, returning an object.  Return:
    NULL for exception;
@@ -374,7 +496,7 @@ try_rich_compare(PyObject *v, PyObject *w, int op)
 	if (v->ob_type != w->ob_type &&
 	    PyType_IsSubtype(w->ob_type, v->ob_type) &&
 	    (f = RICHCOMPARE(w->ob_type)) != NULL) {
-		res = (*f)(w, v, swapped_op[op]);
+		res = (*f)(w, v, _Py_SwappedOp[op]);
 		if (res != Py_NotImplemented)
 			return res;
 		Py_DECREF(res);
@@ -386,7 +508,7 @@ try_rich_compare(PyObject *v, PyObject *w, int op)
 		Py_DECREF(res);
 	}
 	if ((f = RICHCOMPARE(w->ob_type)) != NULL) {
-		return (*f)(w, v, swapped_op[op]);
+		return (*f)(w, v, _Py_SwappedOp[op]);
 	}
 	res = Py_NotImplemented;
 	Py_INCREF(res);
@@ -478,9 +600,7 @@ try_3way_compare(PyObject *v, PyObject *w)
 	/* If both have the same (non-NULL) tp_compare, use it. */
 	if (f != NULL && f == w->ob_type->tp_compare) {
 		c = (*f)(v, w);
-		if (c < 0 && PyErr_Occurred())
-			return -1;
-		return c < 0 ? -1 : c > 0 ? 1 : 0;
+		return adjust_tp_compare(c);
 	}
 
 	/* If either tp_compare is _PyObject_SlotCompare, that's safe. */
@@ -500,9 +620,7 @@ try_3way_compare(PyObject *v, PyObject *w)
 		c = (*f)(v, w);
 		Py_DECREF(v);
 		Py_DECREF(w);
-		if (c < 0 && PyErr_Occurred())
-			return -2;
-		return c < 0 ? -1 : c > 0 ? 1 : 0;
+		return adjust_tp_compare(c);
 	}
 
 	/* Try w's comparison, if defined */
@@ -510,9 +628,11 @@ try_3way_compare(PyObject *v, PyObject *w)
 		c = (*f)(w, v); /* swapped! */
 		Py_DECREF(v);
 		Py_DECREF(w);
-		if (c < 0 && PyErr_Occurred())
-			return -2;
-		return c < 0 ? 1 : c > 0 ? -1 : 0; /* negated! */
+		c = adjust_tp_compare(c);
+		if (c >= -1)
+			return -c; /* Swapped! */
+		else
+			return c;
 	}
 
 	/* No comparison defined */
@@ -567,12 +687,12 @@ default_3way_compare(PyObject *v, PyObject *w)
 	if (w == Py_None)
 		return 1;
 
-	/* different type: compare type names */
-	if (v->ob_type->tp_as_number)
+	/* different type: compare type names; numbers are smaller */
+	if (PyNumber_Check(v))
 		vname = "";
 	else
 		vname = v->ob_type->tp_name;
-	if (w->ob_type->tp_as_number)
+	if (PyNumber_Check(w))
 		wname = "";
 	else
 		wname = w->ob_type->tp_name;
@@ -586,14 +706,12 @@ default_3way_compare(PyObject *v, PyObject *w)
 		Py_uintptr_t)(w->ob_type)) ? -1 : 1;
 }
 
-#define CHECK_TYPES(o) PyType_HasFeature((o)->ob_type, Py_TPFLAGS_CHECKTYPES)
-
 /* Do a 3-way comparison, by hook or by crook.  Return:
-   -2 for an exception;
+   -2 for an exception (but see below);
    -1 if v <  w;
     0 if v == w;
     1 if v >  w;
-   If the object implements a tp_compare function, it returns
+   BUT: if the object implements a tp_compare function, it returns
    whatever this function returns (whether with an exception or not).
 */
 static int
@@ -605,9 +723,22 @@ do_cmp(PyObject *v, PyObject *w)
 	if (v->ob_type == w->ob_type
 	    && (f = v->ob_type->tp_compare) != NULL) {
 		c = (*f)(v, w);
-		if (c != 2 || !PyInstance_Check(v))
-			return c;
+		if (PyInstance_Check(v)) {
+			/* Instance tp_compare has a different signature.
+			   But if it returns undefined we fall through. */
+			if (c != 2)
+				return c;
+			/* Else fall through to try_rich_to_3way_compare() */
+		}
+		else
+			return adjust_tp_compare(c);
 	}
+	/* We only get here if one of the following is true:
+	   a) v and w have different types
+	   b) v and w have the same type, which doesn't have tp_compare
+	   c) v and w are instances, and either __cmp__ is not defined or
+	      __cmp__ returns NotImplemented
+	*/
 	c = try_rich_to_3way_compare(v, w);
 	if (c < 2)
 		return c;
@@ -617,112 +748,6 @@ do_cmp(PyObject *v, PyObject *w)
 	return default_3way_compare(v, w);
 }
 
-/* compare_nesting is incremented before calling compare (for
-   some types) and decremented on exit.  If the count exceeds the
-   nesting limit, enable code to detect circular data structures.
-
-   This is a tunable parameter that should only affect the performance
-   of comparisons, nothing else.  Setting it high makes comparing deeply
-   nested non-cyclical data structures faster, but makes comparing cyclical
-   data structures slower.
-*/
-#define NESTING_LIMIT 20
-
-static int compare_nesting = 0;
-
-static PyObject*
-get_inprogress_dict(void)
-{
-	static PyObject *key;
-	PyObject *tstate_dict, *inprogress;
-
-	if (key == NULL) {
-		key = PyString_InternFromString("cmp_state");
-		if (key == NULL)
-			return NULL;
-	}
-
-	tstate_dict = PyThreadState_GetDict();
-	if (tstate_dict == NULL) {
-		PyErr_BadInternalCall();
-		return NULL;
-	} 
-
-	inprogress = PyDict_GetItem(tstate_dict, key); 
-	if (inprogress == NULL) {
-		inprogress = PyDict_New();
-		if (inprogress == NULL)
-			return NULL;
-		if (PyDict_SetItem(tstate_dict, key, inprogress) == -1) {
-		    Py_DECREF(inprogress);
-		    return NULL;
-		}
-		Py_DECREF(inprogress);
-	}
-
-	return inprogress;
-}
-
-static PyObject *
-check_recursion(PyObject *v, PyObject *w, int op)
-{
-	PyObject *inprogress;
-	PyObject *token;
-	Py_uintptr_t iv = (Py_uintptr_t)v;
-	Py_uintptr_t iw = (Py_uintptr_t)w;
-	PyObject *x, *y, *z;
-
-	inprogress = get_inprogress_dict();
-	if (inprogress == NULL)
-		return NULL;
-
-	token = PyTuple_New(3);
-	if (token == NULL)
-		return NULL;
-
-	if (iv <= iw) {
-		PyTuple_SET_ITEM(token, 0, x = PyLong_FromVoidPtr((void *)v));
-		PyTuple_SET_ITEM(token, 1, y = PyLong_FromVoidPtr((void *)w));
-		if (op >= 0)
-			op = swapped_op[op];
-	} else {
-		PyTuple_SET_ITEM(token, 0, x = PyLong_FromVoidPtr((void *)w));
-		PyTuple_SET_ITEM(token, 1, y = PyLong_FromVoidPtr((void *)v));
-	}
-	PyTuple_SET_ITEM(token, 2, z = PyInt_FromLong((long)op));
-	if (x == NULL || y == NULL || z == NULL) {
-		Py_DECREF(token);
-		return NULL;
-	}
-
-	if (PyDict_GetItem(inprogress, token) != NULL) {
-		Py_DECREF(token);
-		return Py_None; /* Without INCREF! */
-	}
-
-	if (PyDict_SetItem(inprogress, token, token) < 0) {
-		Py_DECREF(token);
-		return NULL;
-	}
-
-	return token;
-}
-
-static void
-delete_token(PyObject *token)
-{
-	PyObject *inprogress;
-
-	if (token == NULL || token == Py_None)
-		return;
-	inprogress = get_inprogress_dict();
-	if (inprogress == NULL)
-		PyErr_Clear();
-	else
-		PyDict_DelItem(inprogress, token);
-	Py_DECREF(token);
-}
-
 /* Compare v to w.  Return
    -1 if v <  w or exception (PyErr_Occurred() true in latter case).
     0 if v == w.
@@ -733,48 +758,18 @@ delete_token(PyObject *token)
 int
 PyObject_Compare(PyObject *v, PyObject *w)
 {
-	PyTypeObject *vtp;
 	int result;
 
-#if defined(USE_STACKCHECK)
-	if (PyOS_CheckStack()) {
-		PyErr_SetString(PyExc_MemoryError, "Stack overflow");
-		return -1;
-	}
-#endif
 	if (v == NULL || w == NULL) {
 		PyErr_BadInternalCall();
 		return -1;
 	}
 	if (v == w)
 		return 0;
-	vtp = v->ob_type;
-	compare_nesting++;
-	if (compare_nesting > NESTING_LIMIT &&
-		(vtp->tp_as_mapping
-		 || (vtp->tp_as_sequence
-		     && !PyString_Check(v)
-		     && !PyTuple_Check(v)))) {
-		/* try to detect circular data structures */
-		PyObject *token = check_recursion(v, w, -1);
-
-		if (token == NULL) {
-			result = -1;
-		}
-		else if (token == Py_None) {
-			/* already comparing these objects.  assume
-			   they're equal until shown otherwise */
-                        result = 0;
-		}
-		else {
-			result = do_cmp(v, w);
-			delete_token(token);
-		}
-	}
-	else {
-		result = do_cmp(v, w);
-	}
-	compare_nesting--;
+	if (Py_EnterRecursiveCall(" in cmp"))
+		return -1;
+	result = do_cmp(v, w);
+	Py_LeaveRecursiveCall();
 	return result < 0 ? -1 : result;
 }
 
@@ -795,7 +790,7 @@ convert_3way_to_object(int op, int c)
 	Py_INCREF(result);
 	return result;
 }
-	
+
 /* We want a rich comparison but don't have one.  Try a 3-way cmp instead.
    Return
    NULL      if error
@@ -845,43 +840,10 @@ PyObject_RichCompare(PyObject *v, PyObject *w, int op)
 	PyObject *res;
 
 	assert(Py_LT <= op && op <= Py_GE);
+	if (Py_EnterRecursiveCall(" in cmp"))
+		return NULL;
 
-	compare_nesting++;
-	if (compare_nesting > NESTING_LIMIT &&
-		(v->ob_type->tp_as_mapping
-		 || (v->ob_type->tp_as_sequence
-		     && !PyString_Check(v)
-		     && !PyTuple_Check(v)))) {
-
-		/* try to detect circular data structures */
-		PyObject *token = check_recursion(v, w, op);
-		if (token == NULL) {
-			res = NULL;
-			goto Done;
-		}
-		else if (token == Py_None) {
-			/* already comparing these objects with this operator.
-			   assume they're equal until shown otherwise */
-			if (op == Py_EQ)
-				res = Py_True;
-			else if (op == Py_NE)
-				res = Py_False;
-			else {
-				PyErr_SetString(PyExc_ValueError,
-					"can't order recursive values");
-				res = NULL;
-			}
-			Py_XINCREF(res);
-		}
-		else {
-			res = do_richcmp(v, w, op);
-			delete_token(token);
-		}
-		goto Done;
-	}
-
-	/* No nesting extremism.
-	   If the types are equal, and not old-style instances, try to
+	/* If the types are equal, and not old-style instances, try to
 	   get out cheap (don't bother with coercions etc.). */
 	if (v->ob_type == w->ob_type && !PyInstance_Check(v)) {
 		cmpfunc fcmp;
@@ -900,7 +862,8 @@ PyObject_RichCompare(PyObject *v, PyObject *w, int op)
 		fcmp = v->ob_type->tp_compare;
 		if (fcmp != NULL) {
 			int c = (*fcmp)(v, w);
-			if (c < 0 && PyErr_Occurred()) {
+			c = adjust_tp_compare(c);
+			if (c == -2) {
 				res = NULL;
 				goto Done;
 			}
@@ -912,7 +875,7 @@ PyObject_RichCompare(PyObject *v, PyObject *w, int op)
 	/* Fast path not taken, or couldn't deliver a useful result. */
 	res = do_richcmp(v, w, op);
 Done:
-	compare_nesting--;
+	Py_LeaveRecursiveCall();
 	return res;
 }
 
@@ -920,18 +883,31 @@ Done:
 int
 PyObject_RichCompareBool(PyObject *v, PyObject *w, int op)
 {
-	PyObject *res = PyObject_RichCompare(v, w, op);
+	PyObject *res;
 	int ok;
 
+	/* Quick result when objects are the same.
+	   Guarantees that identity implies equality. */
+	if (v == w) {
+		if (op == Py_EQ)
+			return 1;
+		else if (op == Py_NE)
+			return 0;
+	}
+
+	res = PyObject_RichCompare(v, w, op);
 	if (res == NULL)
 		return -1;
-	ok = PyObject_IsTrue(res);
+	if (PyBool_Check(res))
+		ok = (res == Py_True);
+	else
+		ok = PyObject_IsTrue(res);
 	Py_DECREF(res);
 	return ok;
 }
 
 /* Set of hash utility functions to help maintaining the invariant that
-	iff a==b then hash(a)==hash(b)
+	if a==b then hash(a)==hash(b)
 
    All the utility functions (_Py_Hash*()) return "-1" to signify an error.
 */
@@ -948,15 +924,7 @@ _Py_HashDouble(double v)
 	 * of mapping keys will turn out weird.
 	 */
 
-#ifdef MPW /* MPW C modf expects pointer to extended as second argument */
-{
-	extended e;
-	fractpart = modf(v, &e);
-	intpart = e;
-}
-#else
 	fractpart = modf(v, &intpart);
-#endif
 	if (fractpart == 0.0) {
 		/* This must return the same hash as an equal int or long. */
 		if (intpart > LONG_MAX || -intpart > LONG_MAX) {
@@ -1009,13 +977,13 @@ _Py_HashPointer(void *p)
 	/* convert to a Python long and hash that */
 	PyObject* longobj;
 	long x;
-	
+
 	if ((longobj = PyLong_FromVoidPtr(p)) == NULL) {
 		x = -1;
 		goto finally;
 	}
 	x = PyObject_Hash(longobj);
-	
+
 finally:
 	Py_XDECREF(longobj);
 	return x;
@@ -1085,21 +1053,23 @@ PyObject_GetAttr(PyObject *v, PyObject *name)
 {
 	PyTypeObject *tp = v->ob_type;
 
-#ifdef Py_USING_UNICODE
-	/* The Unicode to string conversion is done here because the
-	   existing tp_getattro slots expect a string object as name
-	   and we wouldn't want to break those. */
-	if (PyUnicode_Check(name)) {
-		name = _PyUnicode_AsDefaultEncodedString(name, NULL);
-		if (name == NULL)
-			return NULL;
-	}
-	else
-#endif
 	if (!PyString_Check(name)) {
-		PyErr_SetString(PyExc_TypeError,
-				"attribute name must be string");
-		return NULL;
+#ifdef Py_USING_UNICODE
+		/* The Unicode to string conversion is done here because the
+		   existing tp_getattro slots expect a string object as name
+		   and we wouldn't want to break those. */
+		if (PyUnicode_Check(name)) {
+			name = _PyUnicode_AsDefaultEncodedString(name, NULL);
+			if (name == NULL)
+				return NULL;
+		}
+		else
+#endif
+		{
+			PyErr_SetString(PyExc_TypeError,
+					"attribute name must be string");
+			return NULL;
+		}
 	}
 	if (tp->tp_getattro != NULL)
 		return (*tp->tp_getattro)(v, name);
@@ -1129,21 +1099,23 @@ PyObject_SetAttr(PyObject *v, PyObject *name, PyObject *value)
 	PyTypeObject *tp = v->ob_type;
 	int err;
 
-#ifdef Py_USING_UNICODE
-	/* The Unicode to string conversion is done here because the
-	   existing tp_setattro slots expect a string object as name
-	   and we wouldn't want to break those. */
-	if (PyUnicode_Check(name)) {
-		name = PyUnicode_AsEncodedString(name, NULL, NULL);
-		if (name == NULL)
-			return -1;
-	}
-	else 
-#endif
 	if (!PyString_Check(name)){
-		PyErr_SetString(PyExc_TypeError,
-				"attribute name must be string");
-		return -1;
+#ifdef Py_USING_UNICODE
+		/* The Unicode to string conversion is done here because the
+		   existing tp_setattro slots expect a string object as name
+		   and we wouldn't want to break those. */
+		if (PyUnicode_Check(name)) {
+			name = PyUnicode_AsEncodedString(name, NULL, NULL);
+			if (name == NULL)
+				return -1;
+		}
+		else
+#endif
+		{
+			PyErr_SetString(PyExc_TypeError,
+					"attribute name must be string");
+			return -1;
+		}
 	}
 	else
 		Py_INCREF(name);
@@ -1206,32 +1178,42 @@ _PyObject_GetDictPtr(PyObject *obj)
 	return (PyObject **) ((char *)obj + dictoffset);
 }
 
+PyObject *
+PyObject_SelfIter(PyObject *obj)
+{
+	Py_INCREF(obj);
+	return obj;
+}
+
 /* Generic GetAttr functions - put these in your tp_[gs]etattro slot */
 
 PyObject *
 PyObject_GenericGetAttr(PyObject *obj, PyObject *name)
 {
 	PyTypeObject *tp = obj->ob_type;
-	PyObject *descr;
+	PyObject *descr = NULL;
 	PyObject *res = NULL;
 	descrgetfunc f;
+	long dictoffset;
 	PyObject **dictptr;
 
-#ifdef Py_USING_UNICODE
-	/* The Unicode to string conversion is done here because the
-	   existing tp_setattro slots expect a string object as name
-	   and we wouldn't want to break those. */
-	if (PyUnicode_Check(name)) {
-		name = PyUnicode_AsEncodedString(name, NULL, NULL);
-		if (name == NULL)
-			return NULL;
-	}
-	else 
-#endif
 	if (!PyString_Check(name)){
-		PyErr_SetString(PyExc_TypeError,
-				"attribute name must be string");
-		return NULL;
+#ifdef Py_USING_UNICODE
+		/* The Unicode to string conversion is done here because the
+		   existing tp_setattro slots expect a string object as name
+		   and we wouldn't want to break those. */
+		if (PyUnicode_Check(name)) {
+			name = PyUnicode_AsEncodedString(name, NULL, NULL);
+			if (name == NULL)
+				return NULL;
+		}
+		else
+#endif
+		{
+			PyErr_SetString(PyExc_TypeError,
+					"attribute name must be string");
+			return NULL;
+		}
 	}
 	else
 		Py_INCREF(name);
@@ -1241,23 +1223,68 @@ PyObject_GenericGetAttr(PyObject *obj, PyObject *name)
 			goto done;
 	}
 
-	descr = _PyType_Lookup(tp, name);
+	/* Inline _PyType_Lookup */
+	{
+		int i, n;
+		PyObject *mro, *base, *dict;
+
+		/* Look in tp_dict of types in MRO */
+		mro = tp->tp_mro;
+		assert(mro != NULL);
+		assert(PyTuple_Check(mro));
+		n = PyTuple_GET_SIZE(mro);
+		for (i = 0; i < n; i++) {
+			base = PyTuple_GET_ITEM(mro, i);
+			if (PyClass_Check(base))
+				dict = ((PyClassObject *)base)->cl_dict;
+			else {
+				assert(PyType_Check(base));
+				dict = ((PyTypeObject *)base)->tp_dict;
+			}
+			assert(dict && PyDict_Check(dict));
+			descr = PyDict_GetItem(dict, name);
+			if (descr != NULL)
+				break;
+		}
+	}
+
+	Py_XINCREF(descr);
+
 	f = NULL;
-	if (descr != NULL) {
+	if (descr != NULL &&
+	    PyType_HasFeature(descr->ob_type, Py_TPFLAGS_HAVE_CLASS)) {
 		f = descr->ob_type->tp_descr_get;
 		if (f != NULL && PyDescr_IsData(descr)) {
 			res = f(descr, obj, (PyObject *)obj->ob_type);
+			Py_DECREF(descr);
 			goto done;
 		}
 	}
 
-	dictptr = _PyObject_GetDictPtr(obj);
-	if (dictptr != NULL) {
-		PyObject *dict = *dictptr;
+	/* Inline _PyObject_GetDictPtr */
+	dictoffset = tp->tp_dictoffset;
+	if (dictoffset != 0) {
+		PyObject *dict;
+		if (dictoffset < 0) {
+			int tsize;
+			size_t size;
+
+			tsize = ((PyVarObject *)obj)->ob_size;
+			if (tsize < 0)
+				tsize = -tsize;
+			size = _PyObject_VAR_SIZE(tp, tsize);
+
+			dictoffset += (long)size;
+			assert(dictoffset > 0);
+			assert(dictoffset % SIZEOF_VOID_P == 0);
+		}
+		dictptr = (PyObject **) ((char *)obj + dictoffset);
+		dict = *dictptr;
 		if (dict != NULL) {
 			res = PyDict_GetItem(dict, name);
 			if (res != NULL) {
 				Py_INCREF(res);
+				Py_XDECREF(descr);
 				goto done;
 			}
 		}
@@ -1265,12 +1292,13 @@ PyObject_GenericGetAttr(PyObject *obj, PyObject *name)
 
 	if (f != NULL) {
 		res = f(descr, obj, (PyObject *)obj->ob_type);
+		Py_DECREF(descr);
 		goto done;
 	}
 
 	if (descr != NULL) {
-		Py_INCREF(descr);
 		res = descr;
+		/* descr was already increfed above */
 		goto done;
 	}
 
@@ -1291,21 +1319,23 @@ PyObject_GenericSetAttr(PyObject *obj, PyObject *name, PyObject *value)
 	PyObject **dictptr;
 	int res = -1;
 
-#ifdef Py_USING_UNICODE
-	/* The Unicode to string conversion is done here because the
-	   existing tp_setattro slots expect a string object as name
-	   and we wouldn't want to break those. */
-	if (PyUnicode_Check(name)) {
-		name = PyUnicode_AsEncodedString(name, NULL, NULL);
-		if (name == NULL)
-			return -1;
-	}
-	else 
-#endif
 	if (!PyString_Check(name)){
-		PyErr_SetString(PyExc_TypeError,
-				"attribute name must be string");
-		return -1;
+#ifdef Py_USING_UNICODE
+		/* The Unicode to string conversion is done here because the
+		   existing tp_setattro slots expect a string object as name
+		   and we wouldn't want to break those. */
+		if (PyUnicode_Check(name)) {
+			name = PyUnicode_AsEncodedString(name, NULL, NULL);
+			if (name == NULL)
+				return -1;
+		}
+		else
+#endif
+		{
+			PyErr_SetString(PyExc_TypeError,
+					"attribute name must be string");
+			return -1;
+		}
 	}
 	else
 		Py_INCREF(name);
@@ -1317,7 +1347,8 @@ PyObject_GenericSetAttr(PyObject *obj, PyObject *name, PyObject *value)
 
 	descr = _PyType_Lookup(tp, name);
 	f = NULL;
-	if (descr != NULL) {
+	if (descr != NULL &&
+	    PyType_HasFeature(descr->ob_type, Py_TPFLAGS_HAVE_CLASS)) {
 		f = descr->ob_type->tp_descr_set;
 		if (f != NULL && PyDescr_IsData(descr)) {
 			res = f(descr, obj, value);
@@ -1372,8 +1403,12 @@ int
 PyObject_IsTrue(PyObject *v)
 {
 	int res;
+	if (v == Py_True)
+		return 1;
+	if (v == Py_False)
+		return 0;
 	if (v == Py_None)
-		res = 0;
+		return 0;
 	else if (v->ob_type->tp_as_number != NULL &&
 		 v->ob_type->tp_as_number->nb_nonzero != NULL)
 		res = (*v->ob_type->tp_as_number->nb_nonzero)(v);
@@ -1384,13 +1419,11 @@ PyObject_IsTrue(PyObject *v)
 		 v->ob_type->tp_as_sequence->sq_length != NULL)
 		res = (*v->ob_type->tp_as_sequence->sq_length)(v);
 	else
-		res = 1;
-	if (res > 0)
-		res = 1;
-	return res;
+		return 1;
+	return (res > 0) ? 1 : res;
 }
 
-/* equivalent of 'not v' 
+/* equivalent of 'not v'
    Return -1 if an error occurred */
 
 int
@@ -1417,7 +1450,10 @@ PyNumber_CoerceEx(PyObject **pv, PyObject **pw)
 	register PyObject *w = *pw;
 	int res;
 
-	if (v->ob_type == w->ob_type && !PyInstance_Check(v)) {
+	/* Shortcut only for old-style types */
+	if (v->ob_type == w->ob_type &&
+	    !PyType_HasFeature(v->ob_type, Py_TPFLAGS_CHECKTYPES))
+	{
 		Py_INCREF(v);
 		Py_INCREF(w);
 		return 0;
@@ -1507,14 +1543,25 @@ merge_class_dict(PyObject* dict, PyObject* aclass)
 	if (bases == NULL)
 		PyErr_Clear();
 	else {
+		/* We have no guarantee that bases is a real tuple */
 		int i, n;
-		assert(PyTuple_Check(bases));
-		n = PyTuple_GET_SIZE(bases);
-		for (i = 0; i < n; i++) {
-			PyObject *base = PyTuple_GET_ITEM(bases, i);
-			if (merge_class_dict(dict, base) < 0) {
-				Py_DECREF(bases);
-				return -1;
+		n = PySequence_Size(bases); /* This better be right */
+		if (n < 0)
+			PyErr_Clear();
+		else {
+			for (i = 0; i < n; i++) {
+				int status;
+				PyObject *base = PySequence_GetItem(bases, i);
+				if (base == NULL) {
+					Py_DECREF(bases);
+					return -1;
+				}
+				status = merge_class_dict(dict, base);
+				Py_DECREF(base);
+				if (status < 0) {
+					Py_DECREF(bases);
+					return -1;
+				}
 			}
 		}
 		Py_DECREF(bases);
@@ -1574,7 +1621,7 @@ PyObject_Dir(PyObject *arg)
 		PyObject *locals = PyEval_GetLocals();
 		if (locals == NULL)
 			goto error;
-		result = PyDict_Keys(locals);
+		result = PyMapping_Keys(locals);
 		if (result == NULL)
 			goto error;
 	}
@@ -1657,6 +1704,11 @@ PyObject_Dir(PyObject *arg)
 	}
 
 	assert(result);
+	if (!PyList_Check(result)) {
+		PyErr_SetString(PyExc_TypeError,
+			"Expected keys() to be a list.");
+		goto error;
+	}
 	if (PyList_Sort(result) != 0)
 		goto error;
 	else
@@ -1687,12 +1739,12 @@ none_repr(PyObject *op)
 
 /* ARGUSED */
 static void
-none_dealloc(PyObject* ignore) 
+none_dealloc(PyObject* ignore)
 {
 	/* This should never get called, but we also don't want to SEGV if
 	 * we accidently decref None out of existance.
 	 */
-	abort();
+	Py_FatalError("deallocating None");
 }
 
 
@@ -1755,6 +1807,15 @@ _Py_ReadyTypes(void)
 	if (PyType_Ready(&PyType_Type) < 0)
 		Py_FatalError("Can't initialize 'type'");
 
+	if (PyType_Ready(&_PyWeakref_RefType) < 0)
+		Py_FatalError("Can't initialize 'weakref'");
+
+	if (PyType_Ready(&PyBool_Type) < 0)
+		Py_FatalError("Can't initialize 'bool'");
+
+	if (PyType_Ready(&PyString_Type) < 0)
+		Py_FatalError("Can't initialize 'str'");
+
 	if (PyType_Ready(&PyList_Type) < 0)
 		Py_FatalError("Can't initialize 'list'");
 
@@ -1768,27 +1829,13 @@ _Py_ReadyTypes(void)
 
 #ifdef Py_TRACE_REFS
 
-static PyObject refchain = {&refchain, &refchain};
-
-void
-_Py_ResetReferences(void)
-{
-	refchain._ob_prev = refchain._ob_next = &refchain;
-	_Py_RefTotal = 0;
-}
-
 void
 _Py_NewReference(PyObject *op)
 {
-	_Py_RefTotal++;
+	_Py_INC_REFTOTAL;
 	op->ob_refcnt = 1;
-	op->_ob_next = refchain._ob_next;
-	op->_ob_prev = &refchain;
-	refchain._ob_next->_ob_prev = op;
-	refchain._ob_next = op;
-#ifdef COUNT_ALLOCS
-	inc_count(op->ob_type);
-#endif
+	_Py_AddToAllObjects(op, 1);
+	_Py_INC_TPALLOCS(op);
 }
 
 void
@@ -1813,9 +1860,7 @@ _Py_ForgetReference(register PyObject *op)
 	op->_ob_next->_ob_prev = op->_ob_prev;
 	op->_ob_prev->_ob_next = op->_ob_next;
 	op->_ob_next = op->_ob_prev = NULL;
-#ifdef COUNT_ALLOCS
-	op->ob_type->tp_frees++;
-#endif
+	_Py_INC_TPFREES(op);
 }
 
 void
@@ -1826,19 +1871,35 @@ _Py_Dealloc(PyObject *op)
 	(*dealloc)(op);
 }
 
+/* Print all live objects.  Because PyObject_Print is called, the
+ * interpreter must be in a healthy state.
+ */
 void
 _Py_PrintReferences(FILE *fp)
 {
 	PyObject *op;
 	fprintf(fp, "Remaining objects:\n");
 	for (op = refchain._ob_next; op != &refchain; op = op->_ob_next) {
-		fprintf(fp, "[%d] ", op->ob_refcnt);
+		fprintf(fp, "%p [%d] ", op, op->ob_refcnt);
 		if (PyObject_Print(op, fp, 0) != 0)
 			PyErr_Clear();
 		putc('\n', fp);
 	}
 }
 
+/* Print the addresses of all live objects.  Unlike _Py_PrintReferences, this
+ * doesn't make any calls to the Python C API, so is always safe to call.
+ */
+void
+_Py_PrintReferenceAddresses(FILE *fp)
+{
+	PyObject *op;
+	fprintf(fp, "Remaining object addresses:\n");
+	for (op = refchain._ob_next; op != &refchain; op = op->_ob_next)
+		fprintf(fp, "%p [%d] %s\n", op, op->ob_refcnt,
+					    op->ob_type->tp_name);
+}
+
 PyObject *
 _Py_GetObjects(PyObject *self, PyObject *args)
 {
@@ -1876,7 +1937,7 @@ PyTypeObject *_Py_cobject_hack = &PyCObject_Type;
 
 
 /* Hack to force loading of abstract.o */
-int (*_Py_abstract_hack)(PyObject *) = &PyObject_Size;
+int (*_Py_abstract_hack)(PyObject *) = PyObject_Size;
 
 
 /* Python's malloc wrappers (see pymem.h) */
@@ -1884,18 +1945,13 @@ int (*_Py_abstract_hack)(PyObject *) = &PyObject_Size;
 void *
 PyMem_Malloc(size_t nbytes)
 {
-#if _PyMem_EXTRA > 0
-	if (nbytes == 0)
-		nbytes = _PyMem_EXTRA;
-#endif
 	return PyMem_MALLOC(nbytes);
 }
 
 void *
 PyMem_Realloc(void *p, size_t nbytes)
 {
-	/* See comment near MALLOC_ZERO_RETURNS_NULL in pyport.h. */
-	return PyMem_REALLOC(p, nbytes ? nbytes : 1);
+	return PyMem_REALLOC(p, nbytes);
 }
 
 void
@@ -1905,27 +1961,6 @@ PyMem_Free(void *p)
 }
 
 
-/* Python's object malloc wrappers (see objimpl.h) */
-
-void *
-PyObject_Malloc(size_t nbytes)
-{
-	return PyObject_MALLOC(nbytes);
-}
-
-void *
-PyObject_Realloc(void *p, size_t nbytes)
-{
-	return PyObject_REALLOC(p, nbytes);
-}
-
-void
-PyObject_Free(void *p)
-{
-	PyObject_FREE(p);
-}
-
-
 /* These methods are used to control infinite recursion in repr, str, print,
    etc.  Container objects that may recursively contain themselves,
    e.g. builtin dictionaries and lists, should used Py_ReprEnter() and
@@ -1949,7 +1984,7 @@ Py_ReprEnter(PyObject *obj)
 
 	dict = PyThreadState_GetDict();
 	if (dict == NULL)
-		return -1;
+		return 0;
 	list = PyDict_GetItemString(dict, KEY);
 	if (list == NULL) {
 		list = PyList_New(0);
@@ -1991,98 +2026,52 @@ Py_ReprLeave(PyObject *obj)
 	}
 }
 
-/*
-  trashcan
-  CT 2k0130
-  non-recursively destroy nested objects
-
-  CT 2k0223
-  everything is now done in a macro.
-
-  CT 2k0305
-  modified to use functions, after Tim Peter's suggestion.
-
-  CT 2k0309
-  modified to restore a possible error.
-
-  CT 2k0325
-  added better safe than sorry check for threadstate
-
-  CT 2k0422
-  complete rewrite. We now build a chain via ob_type
-  and save the limited number of types in ob_refcnt.
-  This is perfect since we don't need any memory.
-  A patch for free-threading would need just a lock.
-*/
-
-#define Py_TRASHCAN_TUPLE       1
-#define Py_TRASHCAN_LIST        2
-#define Py_TRASHCAN_DICT        3
-#define Py_TRASHCAN_FRAME       4
-#define Py_TRASHCAN_TRACEBACK   5
-/* extend here if other objects want protection */
+/* Trashcan support. */
 
+/* Current call-stack depth of tp_dealloc calls. */
 int _PyTrash_delete_nesting = 0;
 
-PyObject * _PyTrash_delete_later = NULL;
+/* List of objects that still need to be cleaned up, singly linked via their
+ * gc headers' gc_prev pointers.
+ */
+PyObject *_PyTrash_delete_later = NULL;
 
+/* Add op to the _PyTrash_delete_later list.  Called when the current
+ * call-stack depth gets large.  op must be a currently untracked gc'ed
+ * object, with refcount 0.  Py_DECREF must already have been called on it.
+ */
 void
 _PyTrash_deposit_object(PyObject *op)
 {
-	int typecode;
-
-	if (PyTuple_Check(op))
-		typecode = Py_TRASHCAN_TUPLE;
-	else if (PyList_Check(op))
-		typecode = Py_TRASHCAN_LIST;
-	else if (PyDict_Check(op))
-		typecode = Py_TRASHCAN_DICT;
-	else if (PyFrame_Check(op))
-		typecode = Py_TRASHCAN_FRAME;
-	else if (PyTraceBack_Check(op))
-		typecode = Py_TRASHCAN_TRACEBACK;
-	else /* We have a bug here -- those are the only types in GC */ {
-		Py_FatalError("Type not supported in GC -- internal bug");
-		return; /* pacify compiler -- execution never here */
-	}
-	op->ob_refcnt = typecode;
-
-	op->ob_type = (PyTypeObject*)_PyTrash_delete_later;
+	assert(PyObject_IS_GC(op));
+	assert(_Py_AS_GC(op)->gc.gc_refs == _PyGC_REFS_UNTRACKED);
+	assert(op->ob_refcnt == 0);
+	_Py_AS_GC(op)->gc.gc_prev = (PyGC_Head *)_PyTrash_delete_later;
 	_PyTrash_delete_later = op;
 }
 
+/* Dealloccate all the objects in the _PyTrash_delete_later list.  Called when
+ * the call-stack unwinds again.
+ */
 void
 _PyTrash_destroy_chain(void)
 {
 	while (_PyTrash_delete_later) {
-		PyObject *shredder = _PyTrash_delete_later;
-		_PyTrash_delete_later = (PyObject*) shredder->ob_type;
+		PyObject *op = _PyTrash_delete_later;
+		destructor dealloc = op->ob_type->tp_dealloc;
 
-		switch (shredder->ob_refcnt) {
-		case Py_TRASHCAN_TUPLE:
-			shredder->ob_type = &PyTuple_Type;
-			break;
-		case Py_TRASHCAN_LIST:
-			shredder->ob_type = &PyList_Type;
-			break;
-		case Py_TRASHCAN_DICT:
-			shredder->ob_type = &PyDict_Type;
-			break;
-		case Py_TRASHCAN_FRAME:
-			shredder->ob_type = &PyFrame_Type;
-			break;
-		case Py_TRASHCAN_TRACEBACK:
-			shredder->ob_type = &PyTraceBack_Type;
-			break;
-		}
-		_Py_NewReference(shredder);
+		_PyTrash_delete_later =
+			(PyObject*) _Py_AS_GC(op)->gc.gc_prev;
 
+		/* Call the deallocator directly.  This used to try to
+		 * fool Py_DECREF into calling it indirectly, but
+		 * Py_DECREF was already called on this object, and in
+		 * assorted non-release builds calling Py_DECREF again ends
+		 * up distorting allocation statistics.
+		 */
+		assert(op->ob_refcnt == 0);
 		++_PyTrash_delete_nesting;
-		Py_DECREF(shredder);
+		(*dealloc)(op);
 		--_PyTrash_delete_nesting;
 	}
 }
-
-#ifdef WITH_PYMALLOC
-#include "obmalloc.c"
-#endif
