@@ -3,19 +3,13 @@
 
 #include "Python.h"
 
-#ifdef macintosh
-extern char *PyMac_StrError(int);
-#undef strerror
-#define strerror PyMac_StrError
-#endif /* macintosh */
-
 #ifndef __STDC__
 #ifndef MS_WINDOWS
 extern char *strerror(int);
 #endif
 #endif
 
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
 #include "windows.h"
 #include "winbase.h"
 #endif
@@ -131,9 +125,8 @@ PyErr_NormalizeException(PyObject **exc, PyObject **val, PyObject **tb)
 	PyObject *initial_tb = NULL;
 
 	if (type == NULL) {
-		/* This is a bug.  Should never happen.  Don't dump core. */
-		PyErr_SetString(PyExc_SystemError,
-			"PyErr_NormalizeException() called without exception");
+		/* There was no exception, so nothing to do. */
+		return;
 	}
 
 	/* If PyErr_SetNone() was used, the value will have been actually
@@ -160,13 +153,13 @@ PyErr_NormalizeException(PyObject **exc, PyObject **val, PyObject **tb)
 			PyObject *args, *res;
 
 			if (value == Py_None)
-				args = Py_BuildValue("()");
+				args = PyTuple_New(0);
 			else if (PyTuple_Check(value)) {
 				Py_INCREF(value);
 				args = value;
 			}
 			else
-				args = Py_BuildValue("(O)", value);
+				args = PyTuple_Pack(1, value);
 
 			if (args == NULL)
 				goto finally;
@@ -212,7 +205,7 @@ finally:
 void
 PyErr_Fetch(PyObject **p_type, PyObject **p_value, PyObject **p_traceback)
 {
-	PyThreadState *tstate = PyThreadState_Get();
+	PyThreadState *tstate = PyThreadState_GET();
 
 	*p_type = tstate->curexc_type;
 	*p_value = tstate->curexc_value;
@@ -259,22 +252,30 @@ PyErr_NoMemory(void)
 }
 
 PyObject *
-PyErr_SetFromErrnoWithFilename(PyObject *exc, char *filename)
+PyErr_SetFromErrnoWithFilenameObject(PyObject *exc, PyObject *filenameObject)
 {
 	PyObject *v;
 	char *s;
 	int i = errno;
-#ifdef MS_WIN32
+#ifdef PLAN9
+	char errbuf[ERRMAX];
+#endif
+#ifdef MS_WINDOWS
 	char *s_buf = NULL;
+	char s_small_buf[28]; /* Room for "Windows Error 0xFFFFFFFF" */
 #endif
 #ifdef EINTR
 	if (i == EINTR && PyErr_CheckSignals())
 		return NULL;
 #endif
+#ifdef PLAN9
+	rerrstr(errbuf, sizeof errbuf);
+	s = errbuf;
+#else
 	if (i == 0)
 		s = "Error"; /* Sometimes errno didn't get set */
 	else
-#ifndef MS_WIN32
+#ifndef MS_WINDOWS
 		s = strerror(i);
 #else
 	{
@@ -299,42 +300,76 @@ PyErr_SetFromErrnoWithFilename(PyObject *exc, char *filename)
 				(LPTSTR) &s_buf,
 				0,	/* size not used */
 				NULL);	/* no args */
-			s = s_buf;
-			/* remove trailing cr/lf and dots */
-			while (len > 0 && (s[len-1] <= ' ' || s[len-1] == '.'))
-				s[--len] = '\0';
+			if (len==0) {
+				/* Only ever seen this in out-of-mem 
+				   situations */
+				sprintf(s_small_buf, "Windows Error 0x%X", i);
+				s = s_small_buf;
+				s_buf = NULL;
+			} else {
+				s = s_buf;
+				/* remove trailing cr/lf and dots */
+				while (len > 0 && (s[len-1] <= ' ' || s[len-1] == '.'))
+					s[--len] = '\0';
+			}
 		}
 	}
-#endif
-	if (filename != NULL)
-		v = Py_BuildValue("(iss)", i, s, filename);
+#endif /* Unix/Windows */
+#endif /* PLAN 9*/
+	if (filenameObject != NULL)
+		v = Py_BuildValue("(isO)", i, s, filenameObject);
 	else
 		v = Py_BuildValue("(is)", i, s);
 	if (v != NULL) {
 		PyErr_SetObject(exc, v);
 		Py_DECREF(v);
 	}
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
 	LocalFree(s_buf);
 #endif
 	return NULL;
 }
 
 
+PyObject *
+PyErr_SetFromErrnoWithFilename(PyObject *exc, char *filename)
+{
+	PyObject *name = filename ? PyString_FromString(filename) : NULL;
+	PyObject *result = PyErr_SetFromErrnoWithFilenameObject(exc, name);
+	Py_XDECREF(name);
+	return result;
+}
+
+#ifdef Py_WIN_WIDE_FILENAMES
+PyObject *
+PyErr_SetFromErrnoWithUnicodeFilename(PyObject *exc, Py_UNICODE *filename)
+{
+	PyObject *name = filename ? 
+	                 PyUnicode_FromUnicode(filename, wcslen(filename)) : 
+	                 NULL;
+	PyObject *result = PyErr_SetFromErrnoWithFilenameObject(exc, name);
+	Py_XDECREF(name);
+	return result;
+}
+#endif /* Py_WIN_WIDE_FILENAMES */
+
 PyObject *
 PyErr_SetFromErrno(PyObject *exc)
 {
-	return PyErr_SetFromErrnoWithFilename(exc, NULL);
+	return PyErr_SetFromErrnoWithFilenameObject(exc, NULL);
 }
 
 #ifdef MS_WINDOWS 
 /* Windows specific error code handling */
-PyObject *PyErr_SetFromWindowsErrWithFilename(
+PyObject *PyErr_SetExcFromWindowsErrWithFilenameObject(
+	PyObject *exc,
 	int ierr,
-	const char *filename)
+	PyObject *filenameObject)
 {
 	int len;
 	char *s;
+	char *s_buf = NULL; /* Free via LocalFree */
+	char s_small_buf[28]; /* Room for "Windows Error 0xFFFFFFFF" */
 	PyObject *v;
 	DWORD err = (DWORD)ierr;
 	if (err==0) err = GetLastError();
@@ -347,28 +382,99 @@ PyObject *PyErr_SetFromWindowsErrWithFilename(
 		err,
 		MAKELANGID(LANG_NEUTRAL,
 		SUBLANG_DEFAULT), /* Default language */
-		(LPTSTR) &s,
+		(LPTSTR) &s_buf,
 		0,	/* size not used */
 		NULL);	/* no args */
-	/* remove trailing cr/lf and dots */
-	while (len > 0 && (s[len-1] <= ' ' || s[len-1] == '.'))
-		s[--len] = '\0';
-	if (filename != NULL)
-		v = Py_BuildValue("(iss)", err, s, filename);
+	if (len==0) {
+		/* Only seen this in out of mem situations */
+		sprintf(s_small_buf, "Windows Error 0x%X", err);
+		s = s_small_buf;
+		s_buf = NULL;
+	} else {
+		s = s_buf;
+		/* remove trailing cr/lf and dots */
+		while (len > 0 && (s[len-1] <= ' ' || s[len-1] == '.'))
+			s[--len] = '\0';
+	}
+	if (filenameObject != NULL)
+		v = Py_BuildValue("(isO)", err, s, filenameObject);
 	else
 		v = Py_BuildValue("(is)", err, s);
 	if (v != NULL) {
-		PyErr_SetObject(PyExc_WindowsError, v);
+		PyErr_SetObject(exc, v);
 		Py_DECREF(v);
 	}
-	LocalFree(s);
+	LocalFree(s_buf);
 	return NULL;
 }
 
+PyObject *PyErr_SetExcFromWindowsErrWithFilename(
+	PyObject *exc,
+	int ierr,
+	const char *filename)
+{
+	PyObject *name = filename ? PyString_FromString(filename) : NULL;
+	PyObject *ret = PyErr_SetExcFromWindowsErrWithFilenameObject(exc, 
+	                                                             ierr, 
+	                                                             name);
+	Py_XDECREF(name);
+	return ret;
+}
+
+#ifdef Py_WIN_WIDE_FILENAMES
+PyObject *PyErr_SetExcFromWindowsErrWithUnicodeFilename(
+	PyObject *exc,
+	int ierr,
+	const Py_UNICODE *filename)
+{
+	PyObject *name = filename ? 
+	                 PyUnicode_FromUnicode(filename, wcslen(filename)) : 
+	                 NULL;
+	PyObject *ret = PyErr_SetExcFromWindowsErrWithFilenameObject(exc, 
+	                                                             ierr, 
+	                                                             name);
+	Py_XDECREF(name);
+	return ret;
+}
+#endif /* Py_WIN_WIDE_FILENAMES */
+
+PyObject *PyErr_SetExcFromWindowsErr(PyObject *exc, int ierr)
+{
+	return PyErr_SetExcFromWindowsErrWithFilename(exc, ierr, NULL);
+}
+
 PyObject *PyErr_SetFromWindowsErr(int ierr)
 {
-	return PyErr_SetFromWindowsErrWithFilename(ierr, NULL);
+	return PyErr_SetExcFromWindowsErrWithFilename(PyExc_WindowsError,
+						      ierr, NULL);
 }
+PyObject *PyErr_SetFromWindowsErrWithFilename(
+	int ierr,
+	const char *filename)
+{
+	PyObject *name = filename ? PyString_FromString(filename) : NULL;
+	PyObject *result = PyErr_SetExcFromWindowsErrWithFilenameObject(
+						      PyExc_WindowsError,
+						      ierr, name);
+	Py_XDECREF(name);
+	return result;
+}
+
+#ifdef Py_WIN_WIDE_FILENAMES
+PyObject *PyErr_SetFromWindowsErrWithUnicodeFilename(
+	int ierr,
+	const Py_UNICODE *filename)
+{
+	PyObject *name = filename ? 
+	                 PyUnicode_FromUnicode(filename, wcslen(filename)) : 
+	                 NULL;
+	PyObject *result = PyErr_SetExcFromWindowsErrWithFilenameObject(
+						      PyExc_WindowsError,
+						      ierr, name);
+	Py_XDECREF(name);
+	return result;
+}
+#endif /* Py_WIN_WIDE_FILENAMES */
 #endif /* MS_WINDOWS */
 
 void
@@ -448,7 +554,7 @@ PyErr_NewException(char *name, PyObject *base, PyObject *dict)
 	classname = PyString_FromString(dot+1);
 	if (classname == NULL)
 		goto failure;
-	bases = Py_BuildValue("(O)", base);
+	bases = PyTuple_Pack(1, base);
 	if (bases == NULL)
 		goto failure;
 	result = PyClass_New(bases, dict, classname);
@@ -487,18 +593,18 @@ PyErr_WriteUnraisable(PyObject *obj)
 	Py_XDECREF(tb);
 }
 
+extern PyObject *PyModule_GetWarningsModule(void);
 
 /* Function to issue a warning message; may raise an exception. */
 int
 PyErr_Warn(PyObject *category, char *message)
 {
-	PyObject *mod, *dict, *func = NULL;
+	PyObject *dict, *func = NULL;
+	PyObject *warnings_module = PyModule_GetWarningsModule();
 
-	mod = PyImport_ImportModule("warnings");
-	if (mod != NULL) {
-		dict = PyModule_GetDict(mod);
+	if (warnings_module != NULL) {
+		dict = PyModule_GetDict(warnings_module);
 		func = PyDict_GetItemString(dict, "warn");
-		Py_DECREF(mod);
 	}
 	if (func == NULL) {
 		PySys_WriteStderr("warning: %s\n", message);
@@ -524,9 +630,9 @@ PyErr_Warn(PyObject *category, char *message)
 
 /* Warning with explicit origin */
 int
-PyErr_WarnExplicit(PyObject *category, char *message,
-		   char *filename, int lineno,
-		   char *module, PyObject *registry)
+PyErr_WarnExplicit(PyObject *category, const char *message,
+		   const char *filename, int lineno,
+		   const char *module, PyObject *registry)
 {
 	PyObject *mod, *dict, *func = NULL;
 
@@ -566,7 +672,7 @@ PyErr_WarnExplicit(PyObject *category, char *message,
    to make printing of exceptions believe it is a syntax error. */
 
 void
-PyErr_SyntaxLocation(char *filename, int lineno)
+PyErr_SyntaxLocation(const char *filename, int lineno)
 {
 	PyObject *exc, *v, *tb, *tmp;
 
@@ -630,7 +736,7 @@ PyErr_SyntaxLocation(char *filename, int lineno)
 */
 
 PyObject *
-PyErr_ProgramText(char *filename, int lineno)
+PyErr_ProgramText(const char *filename, int lineno)
 {
 	FILE *fp;
 	int i;
@@ -638,14 +744,14 @@ PyErr_ProgramText(char *filename, int lineno)
 
 	if (filename == NULL || lineno <= 0)
 		return NULL;
-	fp = fopen(filename, "r");
+	fp = fopen(filename, "r" PY_STDIOTEXTMODE);
 	if (fp == NULL)
 		return NULL;
 	for (i = 0; i < lineno; i++) {
 		char *pLastChar = &linebuf[sizeof(linebuf) - 2];
 		do {
 			*pLastChar = '\0';
-			if (fgets(linebuf, sizeof linebuf, fp) == NULL)
+			if (Py_UniversalNewlineFgets(linebuf, sizeof linebuf, fp, NULL) == NULL)
 				break;
 			/* fgets read *something*; if it didn't get as
 			   far as pLastChar, it must have found a newline
