@@ -1496,26 +1496,34 @@ static arc arcs_69_0[1] = {
 	{26, 1},
 };
 static arc arcs_69_1[3] = {
-	{25, 2},
-	{147, 3},
+	{147, 2},
+	{25, 3},
 	{0, 1},
 };
 static arc arcs_69_2[1] = {
-	{26, 4},
+	{0, 2},
 };
 static arc arcs_69_3[1] = {
-	{0, 3},
+	{26, 4},
 };
 static arc arcs_69_4[2] = {
-	{147, 3},
+	{13, 5},
 	{0, 4},
 };
-static state states_69[5] = {
+static arc arcs_69_5[1] = {
+	{147, 6},
+};
+static arc arcs_69_6[1] = {
+	{15, 2},
+};
+static state states_69[7] = {
 	{1, arcs_69_0},
 	{3, arcs_69_1},
 	{1, arcs_69_2},
 	{1, arcs_69_3},
 	{2, arcs_69_4},
+	{1, arcs_69_5},
+	{1, arcs_69_6},
 };
 static arc arcs_70_0[2] = {
 	{146, 1},
@@ -1791,7 +1799,7 @@ static dfa dfas[78] = {
 	 "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\002"},
 	{324, "arglist", 0, 8, states_68,
 	 "\000\040\010\060\000\000\000\000\000\000\000\000\000\002\000\140\010\111\023\000"},
-	{325, "argument", 0, 5, states_69,
+	{325, "argument", 0, 7, states_69,
 	 "\000\040\010\000\000\000\000\000\000\000\000\000\000\002\000\140\010\111\023\000"},
 	{326, "list_iter", 0, 2, states_70,
 	 "\000\000\000\000\000\000\000\000\000\000\000\042\000\000\000\000\000\000\000\000"},
